from .init_creds import init_mongo
from bson import ObjectId

db_test = init_mongo(True)
db = init_mongo(False)


def update_socios():
    for socio in db_test.clientes.find():
        active_plan = db_test.planes.find_one({"slug": socio["active_plan"]})
        corporativo = db_test.corporativo.find_one({"slug": socio["plan_corporativo"]})
        settings = {"active_plan": ObjectId(active_plan["_id"] if active_plan else None),
                    "plan_corporativo": ObjectId(corporativo["_id"]) if corporativo else None}
        db_test.clientes.update_one({"_id": socio["_id"]}, {"$set": settings})
        print(socio["cod_client"])


def replicate_db(collection):
    '''
    Replica la base de datos de producción en la base de TEST
       :param str: collection a replicar
       :return: None
       :rtype: None
       '''
    db_test[collection].drop()
    print(collection, "was dropped form dbt")

    print("updating", collection)
    full = db[collection].find({})
    db_test[collection].insert_many(full)


def add_planid():
    for s in db.clientes.find({}):
        plan = db.planes.find_one({"slug": s["active_plan"]})
        db.clientes.update_one({"documento": s["documento"]}, {"$set": {"active_plan_id": plan["_id"]}})


def add_merchants():
    #for c in db.club.find({"asociacion": "SportClub"}):
    db.planes.update_many({"nivel_de_acceso": "Full"}, {"$set": {"merchant_id": ObjectId("617aafd1153d78a8f368dcd5")}})


def move_collection(collection):
    full = db_test[collection].find({})
    db[collection].insert_many(full)




import bson
from zeep import Client
from .init_creds import test, init_mongo
from .format import muvi_print
from .authorization import authorize

from pprint import pprint


class AFIP():
    config = None
    wsaa_client = None
    service_client = None
    ticket = None
    def __init__(self, config_id: bson.ObjectId) -> None:
        """ Método constructor

        :param config_id: id de la configuracion de afip
        :type config_id: bson.ObjectId
        """
        # * Buscar configuracion en db y obtener certificados
        db = init_mongo()
        self.config = db.configuraciones.find_one(config_id)
        #pprint(self.config)
        wsaa_testing = 'https://wsaahomo.afip.gov.ar/ws/services/LoginCms?WSDL'
        wsaa_production = 'https://wsaa.afip.gov.ar/ws/services/LoginCms?WSDL'
        wsdl_testing = 'https://wswhomo.afip.gov.ar/wsfev1/service.asmx?WSDL'
        wsdl_production = 'https://servicios1.afip.gov.ar/wsfev1/service.asmx?WSDL'

        try:
            self.wsaa_client = Client(
                wsaa_production if not(test) and self.config['name'] == 'facturacion-afip-prod' else wsaa_testing)
            print('Instanciando...', wsaa_production)

            self.service_client = Client(
                wsdl_production if not (test) and self.config['name'] == 'facturacion-afip-prod' else wsdl_testing)
            print('Instanciando...', wsdl_production)

        except Exception as e:
            print(e)
            self.wsaa_client = None
            self.service_client = None

    def make_token(self, service: str = 'wsfe') -> None:
        """ Método que genera un xml, lo firma y crea un token, si el merchant para el cual se quiere facturar
        ya tiene un token válido lo reutiliza, caso contrario genera uno nuevo

        :param service: servicio para el cual se quiere crear un token
        :type service: str
        :return: diccionario con datos del token
        :rtype: dict
        """
        ticket = authorize(self.wsaa_client, self.config, service)
        if ticket:
            self.ticket = ticket
        return

    def get_status(self):
        """
        Método dummy para comprobar estado del servidor
        """
        ret = self.service_client.service.FEDummy()
        return ret


    def consultarPuntosVenta(self,merchant:dict) -> list:
        """ Método que informa todos los puntos de venta habilitados para una cuit

        :return: lista de puntos de venta
        :rtype: list
        """
        ret = self.service_client.service.FEParamGetPtosVenta(
            Auth={
                'Token': self.ticket['token'],
                'Sign': self.ticket['sign'],
                'Cuit': merchant['afip']['cuit']
            },
        )
        return ret

    def autorizarComprobante(self, factura: dict,merchant:dict):
        try:
            ret = self.service_client.service.FECAESolicitar(
                Auth={
                    'Token': self.ticket['token'],
                    'Sign': self.ticket['sign'],
                    'Cuit': merchant['afip']['cuit']
                },
                FeCAEReq=factura
            )
            return ret

        except Exception as e:
            muvi_print('warning', '-- Cae no generado --')
            print('Socio: ', factura['FeDetReq']['FECAEDetRequest']['DocNro'])
            muvi_print('error', e)
            return

    def ultimoAutorizado(self, tipo_cbte: int, merchant:dict) -> dict:
        try:
            ret = self.service_client.service.FECompUltimoAutorizado(
                Auth={
                    'Token': self.ticket['token'],
                    'Sign': self.ticket['sign'],
                    'Cuit': merchant['afip']['cuit']
                },
                PtoVta=merchant['afip']['pto_venta'],
                CbteTipo=tipo_cbte
            )
            return ret
        except Exception as e:
            muvi_print('error', e)

    def recuperarComprobante(self, tipo_cbte: int, cbte_nro: int,merchant:dict) -> dict:
        ret = self.service_client.service.FECompConsultar(
            Auth={
                'Token': self.ticket['token'],
                'Sign': self.ticket['sign'],
                'Cuit': merchant['afip']['cuit']
            },
            FeCompConsReq={
                'CbteTipo': tipo_cbte,
                'CbteNro': cbte_nro,
                'PtoVta': merchant['afip']['pto_venta']
            }
        )
        return ret

    def consultarTiposDocumento(self,merchant:dict) -> list:
        """ Método que informa todos los posibles tipos de documentos a los que se les puede generar un comprobante

        :return: lista de tipos de documentos
        :rtype: list
        """
        ret = self.service_client.service.FEParamGetTiposDoc(
            Auth={
                'Token': self.ticket['token'],
                'Sign': self.ticket['sign'],
                'Cuit': merchant['afip']['cuit']
            }
        )
        return ret['ResultGet']['DocTipo']

    def consultarCondicionesIVA(self,merchant:dict) -> list:
        """ Método que informa todas las condiciones IVA disponibles

        :return: lista de condiciones IVA
        :rtype: list
        """
        ret = self.service_client.service.FEParamGetTiposIva(
            Auth={
                'Token': self.ticket['token'],
                'Sign': self.ticket['sign'],
                'Cuit': merchant['afip']['cuit']
            }
        )
        return ret['ResultGet']['IvaTipo']

    def consultarCotizacionMoneda(self, codigo_moneda: str,merchant:dict) -> dict:
        """ Método que informa la cotizacion para una moneda

        :param codigo_moneda: código de la moneda a consultar ('PES' para peso argentino) ver método consultarMonedas
        :type codigo_moneda: str
        :return: _description_
        :rtype: dict
        """
        ret = self.service_client.service.FEParamGetCotizacion(
            Auth={
                'Token': self.ticket['token'],
                'Sign': self.ticket['sign'],
                'Cuit': merchant['afip']['cuit']
            },
            MonId=codigo_moneda
        )
        return ret['ResultGet']

    def consultarMonedas(self,merchant:dict) -> list:
        """ Método que informa las distintas monedas disponibles

        :return: lista de monedas
        :rtype: list
        """
        ret = self.service_client.service.FEParamGetTiposMonedas(
            Auth={
                'Token': self.ticket['token'],
                'Sign': self.ticket['sign'],
                'Cuit': merchant['afip']['cuit']
            }
        )
        return ret['ResultGet']



proj_accesso_cliente = {'token_spice': 1,
                        "token_unauthorized": 1,
                               '_id': 1,
                               "fecha_vigencia": 1,
                               "nivel_de_acceso": 1,
                               "status": 1,

                               "nombre": 1,
                               "apellido": 1,
                               "nacimiento": 1,
                                "active_plan_id": 1,
                        "email": 1,
                        "documento": 1,
                        "plan_corporativo": 1,
                        "plan": 1,
                        "sede": 1
                        }


def create_pipeline_merchant_in_socios(socio_id):
    return [
        {
            '$match': {
                '_id': socio_id
            }
        }, {
            '$lookup': {
                'from': 'planes',
                'localField': 'active_plan_id',
                'foreignField': '_id',
                'as': 'plan'
            }
        }, {
            '$unwind': {'path': '$plan'}
        }, {
            '$lookup': {
                'from': 'merchants',
                'localField': 'plan.merchant_id',
                'foreignField': '_id',
                'as': 'merchant'
            }
        }, {
            '$unwind': {'path': '$merchant'}
        }
    ]


class MuviBase:
    def __init__(self, data=None):
        self.data = data if data else {}
        self._to_update = {}

    def update(self, **kwargs):
        for k, v in kwargs.items():
            if hasattr(self, k):
                setattr(self, k, v)

            if k == 'id':
                self.data['_id'] = v
                continue

            self.data[k] = v
            self._to_update[k] = v



import requests
from pprint import pprint

# ASOCIATE ENVIRONMENT - PRODUCTION
apikey = '700df702fdaee7e441b3d57d712e05cd70687c6bf6ff7898d505132c7b0f1255'
asociate_id = '6006f5bf87d96d2131258ef0'


header = {"Authorization": "Bearer " + apikey,
          "accept-version": "1.0.0",
          "Content-Type": "application/json",
          "Accept": "*/*",
          "Cache-Control": "no-cache",
          "Host": "api.webflow.com",
          "Accept-Encoding": "gzip, deflate, br",
          "Connection": "keep-alive"}


url = 'https://api.webflow.com/'


def get_collections():
    r = requests.get(url + f'sites/{asociate_id}/collections', headers=header)
    return r.json()


def get_schema(collection_id):
    r = requests.get(url + f'collections/{collection_id}', headers=header)
    return r.json()


def get_sites():
    r = requests.get(url + 'sites', headers=header)
    return r.text


def get_items(collection_id):
    r = requests.get(url + f'collections/{collection_id}/items', headers=header)
    response = r.json()
    items = response["items"]
    missing_items = response["total"] - response["count"]
    reached_items = response["count"]
    while missing_items > 0:
        offset = reached_items
        r = requests.get(url + f'collections/{collection_id}/items?offset={offset}', headers=header)
        response = r.json()
        missing_items = missing_items - response["count"]
        reached_items = reached_items + response["count"]
        items.extend(response["items"])
    return items


def get_item(collection_id, item_id):
    r = requests.get(url + f'collections/{collection_id}/items/{item_id}', headers=header)
    response = r.json()
    return response




import base64
from datetime import datetime
from typing import Optional

import iso8601
import mailchimp_transactional as MailchimpTransactional
from bson import ObjectId
from mailchimp_transactional.api_client import ApiClientError

from .dates import today_argentina
from .format import muvi_print
from .init_creds import init_mongo, mailchimp_key, test

mailchimp = MailchimpTransactional.Client(mailchimp_key)


def send_mail_with_attachment(
        files_attachments: list,
        receiver_mail,
        test_mail="matias@muvinai.com",
        global_vars: list = [],
        template: str = 'facturacion',
        brand: dict = {}
):
    print(f"Enviando mail {template}")
    attachment = []
    for file in files_attachments:
        try:
            with open(file['path'], 'rb') as f:
                file_str = f.read()
                file_str = base64.b64encode(file_str)
                file_str = file_str.decode('utf-8')
            muvi_print('info', 'Archivo adjunto procesado con exito')
        except:
            return 'error: el archivo no pudo ser encontrado o procesado'

        attachment.append({
            'content': file_str,
            'name': file['name'].split('/')[-1],
            'type': file['type'] if 'type' in file else 'application/pdf'
        })

    if test:
        to_mail = [{"email": test_mail}]
    else:
        to_mail = [{"email": receiver_mail}]

    msg = {
        'from_email': brand.get('mail_sender_address'),
        'from_name': brand.get('name'),
        'to': to_mail,
        'global_merge_vars': global_vars,
        'attachments': attachment
    }
    try:
        response = mailchimp.messages.send_template({
            'template_name': template,
            'template_content': [],
            'message': msg
        })
        print(response)
        return response[0]
    except ApiClientError as error:
        print('An exception occurred: {}'.format(error.text))
        return 'error'


def send_mail_to_template(receiver: dict, template: str, brand: dict, plan_name: str = "None") -> dict:
    """ Enviar mail con un template determinado

    :param receiver: objeto de cliente del destinatario
    :type receiver: dict
    :param template: nombre del template
    :type template: str
    :param brand: brand
    :type brand: dict
    :param plan_name: nombre del plan, defaults to "None"
    :type plan_name: str, optional
    :return: informacion del mail
    :rtype: dict
    """
    try:
        sdate = receiver["last_subscription_date"].strftime("%d/%m/%Y")

    except AttributeError:
        sdate = iso8601.parse_date(receiver["last_subscription_date"]).strftime("%d/%m/%Y")
    except:
        return {}

    global_vars = [{"name": "nombre", "content": receiver["nombre"]},
                   {"name": "apellido", "content": receiver["apellido"]},
                   {"name": "documento", "content": receiver["documento"]},
                   {"name": "plan", "content": plan_name},
                   {"name": "fecha_subscripcion", "content": sdate}
                   ]
    if brand["name"].lower() != "sportclub":
        global_vars.extend([
            {"name": "logo", "content": brand['images']['mail_logo']},
            {"name": "brand_name", "content": brand["name"]}
        ])
    return send_mail(receiver["email"], global_vars, template, brand=brand)


def send_alert(reciever_mail: str, proceso: str, mensaje: str,
               referencia: str, test_mail="matias@muvinai.com") -> Optional[dict]:
    """ Enviar mensaje de alerta a ignacio@muvinai.com
    :param reciever_mail: email de quien recibe la alerta
    :type reciever_mail: str
    :param proceso: nombre del proceso
    :type proceso: str
    :param mensaje: mensaje
    :type mensaje: str
    :param referencia: referencia
    :type referencia: str
    :param test_mail: test_mail
    :type test_mail: str
    :return: Respuesta de mailchimp o None en caso de error
    :rtype: dict | None
    """

    global_vars = [
        {"name": "proceso", "content": proceso},
        {"name": "mensaje", "content": mensaje},
        {"name": "referencia", "content": referencia}
    ]

    brand = {
        "mail_sender_address": "no-responder@sportclub.com.ar",
        "name": "SportClub"
    }
    return send_mail(reciever_mail, global_vars, template="alertas", brand=brand, test_mail=test_mail)


def send_mail_inactivo(receiver: dict, *, brand: dict, sede: str = None, email_contacto: str = None,
                       test_mail='matias@muvinai.com'):
    """ Enviar mail indicando al cliente que está inactivo.

    :param receiver: documento de cliente del destinatario
    :type receiver: dict
    :param brand: brand
    :type brand: dict
    :param sede: sede
    :type sede: dict
    :param email_contacto: email_contacto
    :type email_contacto: str
    :param test_mail: test_mail
    :type brand: str
    :return: informacion del mail
    :rtype: dict
    """
    db = init_mongo()
    template = 'inactivo'
    global_vars = [{'name': 'nombre', 'content': receiver['nombre']}]

    if brand["name"].lower() != "sportclub":
        template = "inactivo-nosc"
        global_vars.extend([
            {"name": "logo", "content": brand["images"]["mail_logo"]},
            {"name": "brand_name", "content": brand["name"]}
        ])
        return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede,
                         email_contacto=email_contacto, test_mail=test_mail)

    merchant_centrales = [doc['_id'] for doc in db.merchants.find({'negocio_central': True}, {'_id': 1})]
    merchant_centrales.append(ObjectId('6178652dcfb117ad58a2cd3d'))

    if receiver.get('status') == 'baja':
        if receiver.get('plan_corporativo', None):
            template = 'inactivacion-por-baja-corpo'
        elif receiver.get('merchant_id') in merchant_centrales and not receiver.get('seller_merchant_id', None):
            template = 'inactivacion-por-baja-corpo-cadena'
            global_vars.extend([{'name': 'checkout_link', 'content': 'https://www.sportclub.com.ar'}])
        else:
            template = 'inactivacion-por-baja-sede'

    if receiver.get('status') == 'activo':
        if receiver.get('merchant_id') in merchant_centrales and not receiver.get('seller_merchant_id', None):
            template = 'inactivaci-n-autom-tica-cadena-corpo'
            global_vars.extend([
                {'name': 'checkout_link', 'content': f"https://www.sportclub.pagar.club/paso2/{receiver.get('slug')}"}
            ])
        else:
            template = 'inactivacion-automatica-sede'

    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede,
                     email_contacto=email_contacto, test_mail=test_mail)


def send_mail(receiver_mail: str, params: list, template: str, *, brand: dict, sede: dict = None,
              email_contacto: str = None, test_mail="matias@muvinai.com", send_at=None):
    """ Estructura y envía mail

    :param receiver_mail: mail del receptor
    :type receiver_mail: str
    :param params: lista de objetos que son parámetros a pasar al template
    :type params: list
    :param template: nombre del template
    :type template: str
    :param brand: brand
    :type brand: dict
    :param sede: sede
    :type sede: dict
    :param email_contacto: email contacto
    :type email_contacto: str
    :param test_mail: mail del receptor en caso de test
    :type receiver_mail: str
    :param send_at: fecha en la cual se debe enviar el mail
    :type send_at: datetime
    :return: informacion del mail
    :rtype: dict
    """
    print("Enviando mail" + template)

    if 'email_contacto' not in [v['name'] for v in params]:
        if not email_contacto:
            email_contacto_sede = sede.get('contact-email') if sede else None
            email_contacto = email_contacto_sede if email_contacto_sede else brand.get('email_contacto')
        if not email_contacto:
            email_contacto = 'info.socios@sportclub.com.ar'
        params.append({'name': 'email_contacto', 'content': email_contacto})

    msg = {
        "from_email": brand["mail_sender_address"],
        "from_name": brand["name"],
        "to": [{"email": test_mail}] if test else [{"email": receiver_mail}],
        "global_merge_vars": params
    }

    try:
        body = {"template_name": template, "template_content": [], "message": msg}
        if send_at:
            body['send_at'] = send_at.strftime('%Y-%m-%d %H:%M:%S')
        response = mailchimp.messages.send_template(body)
        print(response)
        return response[0]
    except ApiClientError as error:
        print("An exception occurred: {}".format(error.text))
        return {}


def send_mail_carrito_abandonado(receiver_mail: str, client_name: str, plan: dict,
                                 corporativo: dict, cupon_code: str, price: int, *,
                                 brand: dict, sede: dict = None, send_at: datetime = None,
                                 seller_merchant_id: ObjectId = None,
                                 email_contacto: str = None,
                                 test_mail: str = "matias@muvinai.com"):
    """ Enviar mail de carrito abandonado

    :param receiver_mail: email del destinatario
    :type receiver_mail: str
    :param client_name: nombre de cliente del destinatario
    :type client_name: str
    :param plan: plan
    :type plan: dict
    :param corporativo:  corporativo
    :type corporativo: dict
    :param cupon_code: cupon de descuento
    :type cupon_code: str
    :param price: precio del plan
    :type price: int
    :param brand: brand del socio
    :type brand: dict
    :param sede: sede del socio
    :type sede: dict
    :param send_at: fecha en la cual se debe enviar el mail
    :type send_at: datetime
    :param seller_merchant_id: seller_merchant_id
    :type seller_merchant_id: ObjectId
    :param email_contacto: email_contacto
    :type email_contacto: str
    :param test_mail: test_mail
    :type test_mail: str
    :return: informacion del mail
    :rtype: dict
    """
    url_slug = corporativo['slug'] if corporativo else plan['slug']

    base_url = f"https://www.sportclub.pagar.club/paso2/{url_slug}"
    campaign = "utm_source=checkout&utm_medium=email&utm_campaign=carrito_abandonado"

    if cupon_code:
        url = f"{base_url}?code={cupon_code}&{campaign}"
    elif seller_merchant_id:
        url = f"{base_url}?seller_merchant_id={seller_merchant_id}&{campaign}"
    else:
        url = f"{base_url}?{campaign}"

    global_vars = [
        {"name": "corpo", "content": corporativo.get('slug') if corporativo else None},
        {"name": "name", "content": client_name},
        {"name": "plan", "content": plan['name']},
        {"name": "price", "content": price},
        {"name": "the_url", "content": url}
    ]
    if brand["name"] != "SportClub":
        return

    template = "carrito-abandonado"
    return send_mail(receiver_mail, global_vars, template, brand=brand, sede=sede, send_at=send_at,
                     email_contacto=email_contacto, test_mail=test_mail)


def cancel_scheduled_email(email_id: str):
    """ Enviar mail de carrito abandonado

    :param email_id: id del email a cancelar
    :type email_id: str
    :return: informacion del email cancelado
    :rtype: dict
    """
    try:
        response = mailchimp.messages.cancel_scheduled({'id': email_id})
        print(response)
        return response
    except ApiClientError as error:
        print("An exception occurred: {}".format(error.text))
        return {}


def cancel_all_scheduled_email_from(email: str):
    """ Enviar mail de carrito abandonado

    :param email: email del destinatario
    :type email: str
    :return: lista de emails cancelados
    :rtype: list
    """
    canceled = []
    emails = mailchimp.messages.list_scheduled({'to': email})
    if isinstance(emails, list):
        for e in emails:
            canceled.append(cancel_scheduled_email(e['_id']))
    return canceled


def send_mail_cambio_tarjeta(receiver: dict, *, brand: dict, sede: dict = None, email_contacto: str = None):
    """ Enviar mail indicando al cliente que debe cambiar la tarjeta.

            :param receiver: documento de cliente del destinatario
            :type receiver: dict
            :param brand: brand
            :type brand: dict
            :param sede: sede
            :type sede: dict
            :param email_contacto: email_contacto
            :type email_contacto: str
            :return: informacion del mail
            :rtype: dict
            """
    fecha_vigencia = receiver["fecha_vigencia"].strftime("%d/%m/%Y")
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "fecha_vigencia", "content": fecha_vigencia}
    ]

    if brand["name"].lower() == "sportclub":
        template = "pago-rechazado"
    elif brand["name"].lower() == "aranceles":
        return
    elif brand["name"].upper() == "AON":
        template = "cambio_de_tarjeta_aon"
        global_vars = [
            {'name': 'nombre', 'content': receiver['name']},
        ]
    else:
        template = "pago-rechazado-nosc"
        global_vars.extend([
            {"name": "horizontal_white", "content": brand['images']["horizontal_white"]},
            {"name": "image_dark", "content": brand['images']["image_dark"]},
            {"name": "image_light", "content": brand['images']["image_light"]},
            {"name": "brand_name", "content": brand["name"]}
        ])

    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede, email_contacto=email_contacto)


def send_mail_bienvenida(receiver: dict, *, plan: dict, brand: dict, sede: dict = None,
                         email_contacto: str = None, test_mail: str = "ignacio@muvinai.com"):
    """ Enviar mail de bienvenida a la suscripción.

    :param receiver: documento de cliente del destinatario
    :type receiver: dict
    :param plan: plan
    :type plan: dict
    :param brand: brand
    :type brand: dict
    :param sede: sede
    :type sede: dict
    :param email_contacto: email_contacto
    :type email_contacto: str
    :param test_mail: test_mail
    :type test_mail: dict
    :return: informacion del mail
    :rtype: dict
    """
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "apellido", "content": receiver["apellido"]},
        {"name": "documento", "content": receiver["documento"]},
        {"name": "plan", "content": plan["name"]},
        {"name": "fecha_subscripcion", "content": today_argentina().strftime("%d/%m/%Y")},
    ]

    template = "bienvenida"
    if plan["nivel_de_acceso"] == "Flex":
        template = "workclub-bienvenida"

    if brand["name"].lower() != 'sportclub':
        template = "bienvenida-nosc"
        global_vars.extend([
            {"name": "logo_mails", "content": brand['images']['mail_logo']},
            {"name": "brand_name", "content": brand["name"]}
        ])

    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede,
                     email_contacto=email_contacto, test_mail=test_mail)


def send_mail_exitoso(receiver: dict, *, plan: dict, brand: dict, sede: dict = None, email_contacto: str = None):
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "apellido", "content": receiver["apellido"]},
        {"name": "plan", "content": plan["name"]},
        {"name": "fecha_subscripcion", "content": today_argentina().strftime("%d/%m/%Y")}
    ]

    if brand["name"].lower() == "sportclub":
        template = "exitoso"
    else:
        template = "exitoso-nosc"
        global_vars.extend([
            {"name": "logo_mails", "content": brand['images']["mail_logo"]},
            {"name": "brand_name", "content": brand["name"]}
        ])

    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede, email_contacto=email_contacto)


def send_mail_checkout_rechazado(receiver: dict, *, plan: dict, brand: dict, sede: dict = None,
                                 email_contacto: str = None):
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "slug", "content": plan["slug"]}
    ]

    if brand["name"].lower() == "sportlub":
        template = "checkout-rechazado"
    else:
        template = "checkout-rechazado-nosc"
        global_vars.extend([
            {"name": "logo_mails", "content": brand['images']["mail_logo"]},
            {"name": "brand_name", "content": brand["name"]}
        ])

    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede, email_contacto=email_contacto)


def send_mail_pago_en_efectivo(receiver: dict, *, final_price: float, plan_name: str, brand: dict, sede: dict = None,
                               email_contacto: str = None):
    global_vars = [
        {
            "name": "nombre",
            "content": receiver["nombre"],
        }, {
            "name": "precio_total",
            "content": final_price
        }, {
            "name": "plan",
            "content": plan_name
        }
    ]
    if brand["name"].lower() == "sportclub":
        template = "nueva-boleta"
    else:
        template = "nueva-boleta-nosc"
        global_vars.extend([
            {"name": "logo", "content": brand['images']["mail_logo"]},
            {"name": "brand_name", "content": brand["name"]}
        ])
    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede, email_contacto=email_contacto)


def send_mail_pago_pendiente(receiver: dict, *, brand: dict, sede: dict = None,
                             email_contacto: str = None, test_mail="ignacio@muvinai.com"):
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]}
    ]
    if brand["name"].lower() == "sportclub":
        template = "pago-pendiente"
    else:
        template = "pago-pendiente-nosc"
        global_vars.extend([
            {"name": "logo", "content": brand['images']["mail_logo"]},
            {"name": "brand_name", "content": brand["name"]}
        ])
    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede,
                     email_contacto=email_contacto, test_mail=test_mail)


def send_mail_cambio_tarjeta_aon(receiver: dict, *, brand: dict, sede: dict = None, email_contacto: str = None):
    """ Enviar mail indicando al cliente que debe cambiar la tarjeta.

    :param receiver: documento de cliente del destinatario
    :type receiver: dict
    :param brand: brand
    :type brand: dict
    :param sede: sede
    :type sede: dict
    :param email_contacto: email_contacto
    :type email_contacto: str
    :return: informacion del mail
    :rtype: dict
    """
    template = "cambio-de-tarjeta-aon"
    global_vars = [
        {'name': 'nombre', 'content': receiver['name']},
    ]

    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede, email_contacto=email_contacto)


def send_mail_vencimiento_anual(receiver: dict, *, brand: dict, sede: dict = None, email_contacto: str = None):
    """ Enviar mail indicando al cliente que debe cambiar la tarjeta.

    :param receiver: documento de cliente del destinatario
    :type receiver: dict
    :param brand: brand
    :type brand: dict
    :param sede: sede
    :type sede: dict
    :param email_contacto: email_contacto
    :type email_contacto: str
    :return: informacion del mail
    :rtype: dict
    """
    template = "vencimiento-anual"
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "apellido", "content": receiver["apellido"]},
        {"name": "plan", "content": receiver["plan"]["name"]},
        {"name": "fecha_vigencia", "content": receiver["fecha_vigencia"].strftime("%d/%m/%Y")},
    ]
    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede, email_contacto=email_contacto)


def send_mail_cambio_plan_herenecia(receiver: dict, *, brand: dict, sede: dict = None, email_contacto: str = None):
    template = "vencimiento-anual-herencia"
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "apellido", "content": receiver["apellido"]},
        {"name": "precio_mensual", "content": receiver["plan"]['price']},
        {"name": "plan", "content": receiver["plan"]["name"]},
        {"name": "plan_herencia", "content": receiver["plan_herencia"]["name"]},
        {"name": "fecha_vigencia", "content": receiver["fecha_vigencia"].strftime("%d/%m/%Y")},
        {"name": "precio_plan_herencia", "content": receiver['plan_herencia']['price']},
        {"name": "frecuencia", "content": receiver['plan_herencia']['cobro'].lower()}
    ]
    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede, email_contacto=email_contacto)


def send_mail_baja(receiver: dict, *, template: str, brand: dict, sede: dict = None, email_contacto: str = None):
    """ Enviar mail indicando al cliente que está en estado baja.

    :param receiver: documento de cliente del destinatario
    :type receiver: dict
    :param template: nombre del template de mailchimp
    :type template: str
    :param brand: brand
    :type brand: dict
    :param sede: sede
    :type sede: dict
    :param email_contacto: email_contacto
    :type email_contacto: str
    :return: informacion del mail
    :rtype: dict
    """
    planes_url = 'https://www.sportclub.com.ar/#planes'
    global_vars = [
        {"name": "nombre", "content": receiver["nombre"]},
        {"name": "checkout_link", "content": planes_url}
    ]

    return send_mail(receiver["email"], global_vars, template, brand=brand, sede=sede, email_contacto=email_contacto)



import typing

import mercadopago
from bson import ObjectId
from deepdiff import DeepDiff

from dateutil.relativedelta import *

from .dates import (calculate_payment_date, get_periodo, set_next_vigency, today_argentina)
from .format import datetime_parser, json_serial
from .init_creds import init_mongo
from .payments import (create_payment_data, get_card_data_from_payment, get_cards, get_client_price,
                       get_empty_card, update_payment_data)
# from ..service import Cliente

db = init_mongo()


def member_unsubscribe(member: dict, reason, source, unsubscribe_request=False, status_boleta_inactivado='expired'):
    """ Da de baja un socio modificando los parámetros necesarios
       :param member: objeto de cliente a dar de baja
       :type receiver: dict
       :param reason: motivo de la baja
       :type template: str
       :param unsubscribe_request: es True si el cliente es 'baja' y puede seguir ingresando
       :type unsubscribe_request: bool, optional
       :return: None
       :rtype: None
       """

    status = 'baja' if unsubscribe_request else 'inactivo'

    history_event = create_history_event(member, status, source, reason)

    db.clientes.update_one(
        {"_id": member["_id"]},
        {
            "$push": {
                "history": history_event
            },
            "$set": {
                "next_payment_date": None,
                "status": status
            }
        }
    )

    db.boletas.update_many(
        {
            "member_id": member["_id"],
            "status": {
                "$in": ["error", "rejected", "pending_efectivo", "restored"]
            }
        },
        {
            "$set": {
                "status": status_boleta_inactivado
            }
        }
    )

    if not unsubscribe_request:
        plan_db = db.planes.find_one(member['active_plan_id'])
        if plan_db['nivel_de_acceso'].lower() == 'flex':
            cliente = Cliente(member)
            boleta = cliente.crear_boleta('pending_accesos')
            if boleta.final_price != 0:
                if member['status'] == 'baja':
                    boleta.pagar()
                else:
                    boleta.set_status('expired')
                boleta.push_to_db()


def create_history_event(member, event_type, source, reason=None):
    if event_type == 'inactivo':
        event_type = 'inactivacion'

    history_event = {
        'event': event_type,
        'date_created': today_argentina(),
        'source': source
    }
    if reason:
        history_event["reason"] = reason

    if event_type in ['alta', 'baja', 'inactivacion', 'revertir_baja']:
        history_event['plan'] = member["active_plan_id"]

        if 'discounts' in member and member["discounts"] and len(member["discounts"]) > 0:
            history_event['discounts'] = member['discounts']

    elif event_type == "cambio_tarjeta":
        history_event["card_id"] = member["active_card"]

    return history_event


def restore_client_from_payment(payment_id: int, sdk: mercadopago.SDK) -> typing.Union[dict, None]:
    """ Método que crea la estructura de un cliente nuevo a partir de un pago de mercadopago.

    :param payment_id: id del pago a analizar
    :type payment_id: int
    :param sdk: sdk de mercadopago para el cual se creo el pago
    :type sdk: mercadopago.SDK
    :return: diccionario de cliente
    :rtype: dict
    """
    payment_response = sdk.payment().get(payment_id)

    if payment_response['status'] >= 400:
        return

    payment = datetime_parser(payment_response['response'])

    try:
        email = payment['additional_info']['items'][0]['description'].split(
            '-')[2].strip().lower()
    except:
        return

    plan = ObjectId(payment['additional_info']['items'][0]['id'])
    nombre = payment['additional_info']['payer']['first_name']
    apellido = payment['additional_info']['payer']['last_name']
    celular = payment['additional_info']['payer']['phone']['number']

    calle = payment['additional_info']['shipments']['receiver_address']['street_name']
    altura = int(payment['additional_info']['shipments']
                 ['receiver_address']['street_number'])
    localidad = payment['additional_info']['shipments']['receiver_address']['city_name']
    provincia = payment['additional_info']['shipments']['receiver_address']['state_name']

    documento = payment['point_of_interaction']['transaction_data']['subscription_id'].split(
        '-')[-1]

    today = today_argentina()
    f_vigencia_apto = today + relativedelta(days=30)

    socio = {
        'nombre': nombre,
        'apellido': apellido,
        'celular': celular,
        'email': email,
        'documento': documento,
        'domicilio': {
            'calle': calle,
            'altura': str(altura),
            'apto_lote': 'n/a',
            'localidad': localidad,
            'provincia': provincia,
            'código postal': '1000'
        },
        'status': 'activo',
        'payment_ids': [payment['id']],
        'last_payment_id': payment['id'],
        'active_plan_id': plan,
        'cobros_recurrentes': 0,
        'nacimiento': '22/02/2022',
        'preferred_payment_method': 'tarjeta',
        'brand_name': 'SportClub',
        'discounts': [],
        'cards': [],
        'apto_medico': {
            'url': '',
            'status': 'pendiente',
            'fecha_vigencia': f_vigencia_apto.replace(day=f_vigencia_apto.day, hour=23, minute=59, second=59)
        },
        'sportaccess_id': f"MUVI-{documento}",
        'poi': {
            'installments': payment['installments'],
            'payment_reference': payment['id']
        }
    }

    # * Dates

    plan = db.planes.find_one({'_id': plan})
    npd = calculate_payment_date(today.day, plan['cobro'], get_periodo(today))
    socio['next_payment_date'] = npd
    socio['fecha_vigencia'] = set_next_vigency(npd)
    socio['last_subscription_date'] = today
    socio['period_init_day'] = today.day

    # * Customer mp

    customer_response = sdk.customer().search({'email': email})['response']
    if customer_response['paging']['total'] == 1:
        customer = customer_response['results'][0]
        customer_id = customer['id']
    else:
        customer_data = {
            "email": email,
            "first_name": nombre,
            "last_name": apellido,
            "phone": {"area_code": None, "number": celular},
            'identification': {'type': 'DNI', 'number': documento},
            'address': {
                'street_name': calle,
                'street_number': altura,
            }
        }
        customer = sdk.customer().create(customer_data)['response']
        if 'email' in customer.keys():
            customer_id = customer['id']
        else:
            customer_id = 'error'

    socio['mercadopago_id'] = customer_id

    # * History

    history_event = create_history_event(
        member=socio, event_type='alta', source='checkout')
    socio['history'] = [history_event]

    # * Plan corporativo

    if plan['price'] == payment['transaction_amount']:
        socio['plan_corporativo'] = None
        plan['corporativo'] = None
    else:
        abandoned = db.abandoned.find_one({'documento': documento})
        try:
            socio['plan_corporativo'] = abandoned['corpo_id']
        except:
            # * No hay forma de recuperar el corporativo que le aplicó un descuento
            return

    return socio


def create_boleta_from_payment(payment_id: int) -> None:
    """ Método que crea una boleta a partir de un payment_id. 
    Para que funcione debe el existir el socio y debe tener el payment_id en su array de payments

    :param payment_id: payment id de mercadopago
    :type payment_id: int
    """

    pipeline = [
        {'$match': {'payment_ids': payment_id}},
        {'$lookup': {
            'from': 'planes',
            'as': 'plan',
            'let': {'active_plan': '$active_plan_id', 'corpo': '$plan_corporativo'},
            'pipeline': [
                {'$project': {
                    'merchant_id': 1,
                    'price': 1
                }},
                {'$match': {
                    '$expr': {
                        '$eq': ['$$active_plan', '$_id']
                    }
                }},
                {'$lookup': {
                    'from': 'corporativo',
                    'as': 'corporativo',
                    'pipeline': [
                        {'$project': {
                            'porcentaje-descuento-empleado': 1,
                            'name': 1
                        }},
                        {'$match': {
                            '$expr': {
                                '$eq': ['$$corpo', '$_id']
                            }
                        }}
                    ]
                }},
                {'$lookup': {
                    'from': 'merchants',
                    'as': 'merchant',
                    'let': {'merchant_id': '$merchant_id'},
                    'pipeline': [
                        {'$project': {
                            'keys.access_token': 1
                        }},
                        {'$match': {
                            '$expr': {
                                '$eq': ['$$merchant_id', '$_id']
                            }
                        }}
                    ]
                }},
                {'$addFields': {
                    'access_token': {
                        '$first': '$merchant.keys.access_token'
                    },
                    'corporativo': {
                        '$first': '$corporativo'
                    }
                }},
                {'$project': {
                    'merchant': 0,
                    'plan_corporativo': 0
                }}
            ]
        }},
        {'$unwind': {'path': '$plan'}},
        {'$project': {
            'documento': 1,
            'cards': 1,
            'discounts': 1,
            'active_plan_id': 1,
            'payment_ids': 1,
            'plan': 1,
            'access_token': '$plan.access_token',
            'corporativo': '$plan.corporativo'
        }},
        {'$project': {
            'plan.access_token': 0,
            'plan.corporativo': 0
        }}
    ]

    client = list(db.clientes.aggregate(pipeline))[0]

    print(f"--- User {client['documento']} ---")
    print('\n')

    sdk = mercadopago.SDK(client['access_token'])

    payment = datetime_parser(sdk.payment().get(payment_id)['response'])

    corpo_discount = 0
    if 'corporativo' in client and client['corporativo']:
        corpo_discount = client['corporativo']['porcentaje-descuento-empleado']

    prices = get_client_price(client=client, corpo_discount=corpo_discount,
                              plan_price=client['plan']['price'], sum_access=False)

    boleta = create_payment_data(_client=client, prices=prices,
                                 merchant_id=client['plan']['merchant_id'], source='checkout3')

    payment_result = {
        'status': payment['status'],
        'status_detail': payment['status_detail'],
        'id': payment['id'],
        'date_created': payment['date_created']
    }

    cards = get_cards(client)
    try:
        card = get_card_data_from_payment(payment)
    except:
        card = cards[0] if cards != [] else get_empty_card()

    update_payment_data(payment_result=payment_result,
                        payment_data=boleta, card_data=card)
    db.boletas.update_one({'_id': boleta['_id']}, {'$set': boleta})
    

def version_control(old_doc, new_doc, current_user, source="API"):
    """
    Funcion que implementa el control de versiones para los clientes de la base de datos(MongoDB).

    Parameters:
        old_doc (dict): el documento original en la base de datos.
        new_doc (dict): el documento modificado que se va a almacenar en la base de datos.
        current_user (str): representa al usuario que realizó la modificación.
        source (str): la fuente de la modificación (API, web, base de datos, etc.). Por defecto es "API".

    Returns:
        vc: un objeto de diccionarios que representan los cambios realizados en el documento, si hay muchos cambios se añidan en un lista (changes).
    """
    old_doc = json_serial(old_doc)
    new_doc = json_serial(new_doc)

    def ddiff_obj_parser(s):

        a = s.split()[0]
        a = a.strip('<').strip('root')
        # print(parse_string(a))
        a = a.replace('][','.').replace("[", "").replace("]", "").replace("'", "").replace(",", ".").replace(" ", "")
        return a

    # Validar los parámetros de entrada
    if not isinstance(old_doc, dict):
        raise TypeError("El documento original debe ser un diccionario.")
    if not isinstance(new_doc, dict):
        raise TypeError("El documento modificado debe ser un diccionario.")

    vc = {}
    # Comparar los documentos y registrar los cambios
    ddiff = DeepDiff(old_doc, new_doc, view='tree', ignore_order=True).to_dict()

    if ddiff:
        vc = {
            "source": source,
            "modified_at": today_argentina(),
            "modified_by": current_user,
            "changes": []
        }
        for change_type, items in ddiff.items():
            if change_type == 'iterable_item_removed':
                for change in items:
                    dot_notation_pos = ddiff_obj_parser(str(change))
                    if ('history' or 'version_control') not in dot_notation_pos:
                        vc['changes'].append({
                            "field": dot_notation_pos,
                            "previous_value": change.t1,
                            "new_value": None,
                            "change_type": change_type,
                        })
            elif change_type == 'iterable_item_added':
                for change in items:
                    dot_notation_pos = ddiff_obj_parser(str(change))
                    if ('history' or 'version_control') not in dot_notation_pos:
                        vc['changes'].append({
                            "field": dot_notation_pos,
                            "previous_value": None,
                            "new_value": change.t2,
                            "change_type": change_type,

                        })
            elif change_type == 'dictionary_item_removed':
                for change in items:
                    dot_notation_pos = ddiff_obj_parser(str(change))
                    if ('history' or 'version_control') not in dot_notation_pos:
                        vc['changes'].append({
                            "field": dot_notation_pos,
                            "previous_value": change.t1,
                            "new_value": None,
                            "change_type": change_type,

                        })
            elif change_type == 'dictionary_item_added':
                for change in items:
                    dot_notation_pos = ddiff_obj_parser(str(change))
                    if ('history' or 'version_control') not in dot_notation_pos:
                        vc['changes'].append({
                            "field": dot_notation_pos,
                            "previous_value": None,
                            "new_value": change.t2,
                            "change_type": change_type,

                        })
            elif change_type in ['values_changed', 'type_changes']:
                for change in items:
                    dot_notation_pos = ddiff_obj_parser(str(change))
                    if ('history' or 'version_control') not in dot_notation_pos:
                        vc['changes'].append({
                            "field": dot_notation_pos,
                            "previous_value": change.t1,
                            "new_value": change.t2,
                            "change_type": change_type,
                        })

    if 'changes' in vc.keys() and vc['changes'] != []:
        return datetime_parser(vc)
    else:
        return None



import os
import json
import pymongo
import gspread
from cryptography.fernet import Fernet
from dotenv import load_dotenv
import mercadopago

load_dotenv()
mongo_user = os.getenv('MONGO_USER')
mongo_pass = os.getenv('MONGO_PASS')
flask_secret = os.getenv('SECRET_KEY')
mailchimp_key = os.getenv('MAILCHIMP_CLIENT')
sportaccess_key = os.getenv('SPORTACCESS_PASSWORD')
api_key = os.getenv('API_KEY')
sportclub_id = os.getenv('SPORTCLUB_ID')

try:
    test = os.getenv('TEST').lower() == 'true'
except AttributeError:
    test = False

gc = None
db_prod = None
db_test = None
db_prod_arg_patea = None
db_test_arg_patea = None
sdk_test = None
sdk_prod = None

test_str = "TEST" if test else "PRODUCCION"
print("Environment: " + test_str)


def init_gspread():
    """ Inicializar gspread (si no fue inicializado anteriormente)

    :return: gspread
    :rtype: Client
    """
    global gc
    if gc is not None:
        return gc   # gc ya esta instanciada

    encrypted = b'gAAAAABhwj0N7xSHt0HdNkQA2arqrs4pDHQMKI4u5CwPJsTT1QZvpAk1MV6eI8Kdyf66C4YJYoDWmtTEMYICbZT0C0kZ2qr6' \
                b'Zb2_1v3C1rByMrRDzfnEAcaqCfVyqQGcaMeIn7eCNiDtWA4NLGO0qH8cPAyEV6JTnJH3nljSlo6cYDKwvD0f6-zRrqOj5qdN4' \
                b'HESJuDGgR3ZDcjFEnPGO-N_wMewtm76gkeJlR-tITOcOQp12f0N6n4WcI4lznOIGMpFzbQ94TL1-NuMSzG_ujTCZEXpSc9UgvdP' \
                b'Lxx0eqM_rOm2zly2yX1oxLKSjISCjDviZ5f59AUuwXf90w_PFbSjaizNP5ZgTZ_DdxVAfiFQR1ZJryHcKz4_ISYHp1zp70kmpI' \
                b'O3QZFffnD6UsUh8jxqYhJoBfc9UnTeY2k7pDPJElfuU98QlftsFxawytLGEIgrm0YzPWUguskv397MlA1nhEtk3iGgkX21NPuH' \
                b'8NkdqXMfGmtV6Wz1Atwq4t-UtHZTKJ3XUEdbU-51C36BdfiGh-F9P7I3U0_NWx9ts0cvyenSSnxjfntH1qE7MOH0-J3Np9oPSHm' \
                b'UkFPzy0ZUTXdO76EKi0ZRK4b4l8s9KD8MjI5i37oP7U-fmQ0d_kudEkDlY1AZxJMO8-YhG7DPvblaP9TOx6lYbzxdG7yoeQseCO' \
                b'NKkgcZADAbliFgUGRmg2n692CkIF64zrP2IgQ_aqLznp4CxI1GOXXfsT8dzYsBzM9bdazYNZ-n7Hgmo_kinpU5N8iVqgZiGV34' \
                b'HEOcSsPbCPhS7dcNph448SRYiGhG2S2GhKXwLfWRKf3UWAHU7TIAJw-i3XiLGc_uZ7b2PglASn7iPJI-uJS1A0WPTfdssfvHl_i' \
                b'1Zd4k2cXNXlL1w_h_7RloyUEEMthnqBqIYT80suixg_krXScBOSZrvIVEP5mjdVgWenaPvIAPUlFbsYwo34448ff2vsh24D4JKH' \
                b'tJ-vZfE-2kyZUQecWT1BistV5ZiNw-kKO5FfRLuA3YyoiIrw0Wb6dwCv4sc9dq1B82vr-RYCoZsCyuJYLKS7HwdXpJWERFTPFf' \
                b'X_jWKwnRuQ9RWS4uG7r3o1iFWNfuwM05Hwo15O_391acG6P0YExH-0FNBVNHDj8DoyXOEJ_dALWxVukLIrK9wykefBPxKF4ej8' \
                b'3gEk52nriSkX33uLeeE2CcRGbsxbI8SSgaI1beT2Ivxdh0C1QJnZ5tXDIKzi3vtB0GnjGkuOZZ9vsMoG4aGUTlXxxPoOzsVre' \
                b'MdvyC6nOU3TgklNv-EXSav9lPUUgoFl2WOu7YSk64-4LHOSde22q9is0jQu5UakBpESmtRNmfMWxF5MqWjxNxJcqSfyv_DSDS' \
                b'Srf9l48Nsb6cIUt-LHdq7CIlrHbkNRlZ_X_p_iiaOL5Em8E979ldzpeByvAozFHKWRkWFbfeleS11gFgDMx8ItiiQizml_VyD' \
                b'kJi_zi7Mm7mUX7DO7x2vpdI_dxyG-yJMlFQLYJ3q4DrWOABbK1zBQJ-7ZIJDJo003gCohfuCWkyO_QGcUiiWFlJ-Cbe7gO3Bh' \
                b'2yztkP53vPt-af222imVtI_xgWT3_jjLY7m6-_870tWB4NEJ_c8UlKiAdV4Ri9vSmcBOiWMa77nCxjPDSQSe5B1fk22ztENBg' \
                b'LngdvmQAYvo-Ck8lOSkkuraD5_9IZpdOmfVB8klMFldctfNqdFSUYGlwRnm4lWLJobZGVWvQ-ozZfBb-x7rswII6EyjOdTKhe' \
                b'2ZmmWuzBLzBGRQt44hCbTSYHO9schkM8nA6FzoGT8Yx4m-H06F_q9fnIMewADoyLkPjhLkWx5pNJFKNgE3ee5NVmjdBps0_v5' \
                b'OkoxTEuj-NgNWfEwD720i9Im3nnRX0ltEMzk80m_JaUIi8-ltKDtgbuzTbpqkV4REP805cZHOupLwhuijmUQunCd6WlW_jT9UD' \
                b'l5_mw6M6B8rT3yoW5lmZnR1QqbgcCpGUZqul_6wptTDCaHBk4sDPMZBJ-Dl4NRxEh9vK2_uJS-En5c5-t1Zt5UwQ-jYIFRJ-lz' \
                b'717ZEm-w3g-_ab_1lpC8LPWVYbIXmqI2TYXjukjJbKhTWHul4wKbxp94SN0TCDhYvDZrWOjZxMF2HkRxBUOWT2sDADet0BNxFV' \
                b'v8CXZO7G1_3qiSZP-r2pCy19fZ9jtyP5bcFv7TqQatPSg1R2ayhMbFwZfvBlVznuVAvgK0gecJaNYAe5c-LQu2KmVuQiaH-Pl7' \
                b'd3YPv3ZvnmXqgmXIEyy2ZRLX7nmtleSMH0qYn41PolZO_q3uK9HplK4zlzoMTHIY2cTUEg-4Hs-B8s1w59W692GV16ii2NGA0k' \
                b'GF6RYuEZMMXg6N0SelSAQb6ZOWKIK1HAQ5aOsy1LrIB5xFkv4U54PrcMWsLa3NYwktM-_pRLbKPnQNvbjG62tGmV9tww0AzoEg' \
                b'usfK9D4mOZTEpE230Zqn2L2VrcHa9-vFJ21hpURkNlPs5ttMGN8TE44q9vMXIq6pJBl8w9_yXeipe0hzylJ1b81_8tFtRZXuzo' \
                b'IZ5HD-b4Rw-xvrxpMCmw0tnOo54pdcI7hKdzaNv2hyu0y41B1ektgCHh033az7zyJf72u_an_LoZlq0IINWBeUMeXI552ohIwl' \
                b'M4aDgei_LhPTluR9kDkxFeSkxhHEQA6K__MJAq8M8HBp3xIQF3QeA7tNDp9n5S1OBGBgHB8CFjc2BM81pEuOSNKUd55QPZcy-_' \
                b'SnKQ0Pu6s7CJBd-qVcLzNYMVaUvyg9-LzmozOpSOxZGwvlaUVtL3k1Ewdbdv6Izsagvf_YYJrOu4y1MRCZ1XhnUwIM0iwWcCcL' \
                b'N2aEwcM_a0mMAcMTylTyVeqmmjHFFK6pC3XsABOHxvhIaG4I3erhHXfoCT7x9Nn-wb9NtxCjb62-3ii6cKMklTIEIQ-C14dMwe' \
                b'te79kiKz75m_8c6WuOZH8yKWl4uzK3QQ0HW0HTVvMHU5gNzcs8HH4__fUjp4Uhir2blDbDwpWW3O2LCb3Gbk0cNcLDKlLwUijk' \
                b'BQPP8fJgN_z7AhoA2gZwapY8ugXLSPsOI3K08Ualk33YVwdbgE4FIjILndOp0sUY3IWd_TCV9BU7tE0ZdncfU3h2XyIOhM='

    key = os.getenv("CRYPTO").encode('utf-8')

    fernet = Fernet(key)
    gspread_credentials = fernet.decrypt(encrypted).decode()
    gc = gspread.service_account_from_dict(json.loads(gspread_credentials))
    return gc


def init_mongo(_test: bool = None):
    """ Inicializar la base de mongo. Si no recibe ningun parametro inicializa la\
        base de datos segun la variable de entorno test.

    Es posible inicializar las bases de produccion y de test al mismo tiempo de la siguiente manera:
        
        >>> db_prod = init_mongo(False)
        >>> db_test = init_mongo(True)

    :param _test: Indica la base a inicializar (True para test, False para produccion), defaults to None
    :type _test: bool, optional
    :return: base de mongo
    :rtype: Database
    """    
    global db_prod
    global db_test
    if _test is None:
        _test = test

    if _test:
        if db_test is not None: return db_test # db_test ya esta instanciada
        client = pymongo.MongoClient(f"mongodb+srv://{mongo_user}:{mongo_pass}@cluster0.sf15y.mongodb.net/checkout",
                                     tlsAllowInvalidCertificates=True,
                                     retryWrites=True,
                                     w="majority")
        db_test = client.checkout
        return db_test

    if db_prod is not None: return db_prod  # db_prod ya esta instanciada
    client = pymongo.MongoClient(f"mongodb+srv://{mongo_user}:{mongo_pass}@cluster0.sf15y.mongodb.net/sportclub_prod",
                                 tlsAllowInvalidCertificates=True,
                                 retryWrites=True,
                                 w="majority")
    db_prod = client.sportclub_prod
    return db_prod


def init_mercadopago(token: str = None) -> mercadopago.SDK:
    """ Inicializar la sdk de mercadopago. Si no recibe ningun parametro\
        inicializa la sdk segun la variable de entorno test. 
    
    Es posible inicializar una sdk de produccion y otra de test al mismo tiempo de la siguiente manera:
        
        >>> sdk_prod = init_mercadopago(False)
        >>> sdk_test = init_mercadopago(True)

    :param token: access token para acceder a la sdk de mercadopago, defaults to None
    :type _test: str, optional
    :return: sdk de mercadopago
    :rtype: mercadopago.SDK
    """
    global sdk_prod
    global sdk_test
    if token is None:
        if sdk_test is not None:
            return sdk_test # sdk_test ya esta instanciada
        access_token_test = os.getenv("MP_ACCESS_TOKEN_TEST")
        sdk_test = mercadopago.SDK(access_token_test)
        return sdk_test
    
    if sdk_prod is not None:
        return sdk_prod # sdk_prod ya esta instanciada
    sdk_prod = mercadopago.SDK(token)
    return sdk_prod


def init_mongo_CR20():
    """ Inicializar la base de mongo de test - CR20.

    :return: base de mongo
    :rtype: Database
    """
    client = pymongo.MongoClient(f"mongodb+srv://{mongo_user}:{mongo_pass}@cluster0.sf15y.mongodb.net/CR20_test",
                                 tlsAllowInvalidCertificates=True,
                                 retryWrites=True,
                                 w="majority")
    db_test = client.CR20_test
    return db_test


def init_mongo_arg_patea(_test: bool = None):
    """ Inicializar la base de mongo para Argentina Patea. Si no recibe ningun parametro inicializa la\
        base de datos segun la variable de entorno test.

    Es posible inicializar las bases de produccion y de test al mismo tiempo de la siguiente manera:

        >>> db_prod_arg_patea = init_mongo_arg_patea(False)
        >>> db_test_arg_patea = init_mongo_arg_patea(True)

    :param _test: Indica la base a inicializar (True para test, False para produccion), defaults to None
    :type _test: bool, optional
    :return: base de mongo
    :rtype: Database
    """
    global db_prod_arg_patea
    global db_test_arg_patea
    if _test is None:
        _test = test

    if _test:
        if db_test_arg_patea is not None: return db_test_arg_patea  # db_test ya esta instanciada
        client = pymongo.MongoClient(f"mongodb+srv://{mongo_user}:{mongo_pass}@cluster0.sf15y.mongodb.net/arg_patea_test",
                                     tlsAllowInvalidCertificates=True,
                                     retryWrites=True,
                                     w="majority")
        db_test_arg_patea = client.arg_patea_test
        return db_test_arg_patea

    if db_prod_arg_patea is not None: return db_prod_arg_patea  # db_prod ya esta instanciada
    client = pymongo.MongoClient(f"mongodb+srv://{mongo_user}:{mongo_pass}@cluster0.sf15y.mongodb.net/arg_patea_prod",
                                 tlsAllowInvalidCertificates=True,
                                 retryWrites=True,
                                 w="majority")
    db_prod_arg_patea = client.arg_patea_prod
    return db_prod_arg_patea


from muvinai.utilities.init_creds import init_mongo
from muvisdk.SDK import SDK
from .brand import Brand
from .sede import Sede

from bson import ObjectId
from typing import Optional


class MerchantsIterator:
    def __init__(self, merchants, cadena):
        self.merchants = merchants
        self.index = 0
        self.cadena = cadena

    def __next__(self):
        index = self.index
        if index == len(self.merchants):
            raise StopIteration

        self.index += 1
        return Merchant(self.merchants[index])


class Merchants:
    def __init__(self, brand_id=None):
        db = init_mongo()
        query = {}
        if brand_id:
            query['brand_id'] = brand_id
        self.merchants = list(db.merchants.find(query))
        self.cadena = db.merchants.find_one(ObjectId('6178652dcfb117ad58a2cd3d'))

    def __iter__(self):
        return MerchantsIterator(self.merchants, self.cadena)


class Merchant:
    _cadena_db = None

    @classmethod
    def _get_cadena_db(cls):
        if not cls._cadena_db:
            db = init_mongo()
            cls._cadena_db = db.merchants.find_one(ObjectId('6178652dcfb117ad58a2cd3d'))
        return cls._cadena_db

    @classmethod
    def get_cadena(cls):
        cadena_db = cls._get_cadena_db()
        return Merchant(cadena_db)

    @classmethod
    def search(cls, brand_id=None):
        return Merchants(brand_id=brand_id)

    def __init__(self, merchant_db, brand: Brand = None, sede: Sede = None):
        self.merchant_db = merchant_db
        self.id = merchant_db['_id']
        self.name = merchant_db['name']
        self.brand_id = merchant_db['brand_id']
        self.sede_principal = merchant_db.get('sede_principal')
        self.credentials = merchant_db['credentials']
        self.application_fee: dict = {k.lower(): v for k, v in merchant_db.get('application_fee', {}).items()}
        self.planes_multinegocio = merchant_db.get('planes_multinegocio', [])
        self.afip = merchant_db.get('afip')
        self.brand = brand
        self.sede = sede
        cadena_db = self._get_cadena_db()
        self.sdk_decidir = SDK(self.merchant_db, cadena_db, 'decidir', marketplace=False)
        self.sdk_decidir_marketplace = SDK(self.merchant_db, cadena_db, 'decidir', marketplace=True)
        self.sdk_mercadopago = SDK(self.merchant_db, cadena_db, 'mercadopago', marketplace=False)
        self.sdk_mercadopago_marketplace = SDK(self.merchant_db, cadena_db, 'mercadopago', marketplace=True)
        self.application_id = self.credentials['mercadopago'].get('application_id')
        self.plans_ids = None

    @property
    def activate_facturacion(self) -> bool:
        return self.afip is not None and self.afip.get('activate_facturacion', False)

    def get_brand(self) -> Brand:
        if not self.brand:
            db = init_mongo()
            self.brand = Brand(db.brands.find_one(self.brand_id))
        return self.brand

    def get_sede(self) -> Sede:
        if not self.sede:
            db = init_mongo()
            self.sede = Sede(db.club.find_one(self.sede_principal))
        return self.sede

    def get_plans_ids(self) -> list:
        if not self.plans_ids:
            db = init_mongo()
            self.plans_ids = [p['_id'] for p in db.planes.find({'merchant_id': self.id}, {'_id': 1})]
        return self.plans_ids

    def get_planes_multinegocio(self) -> list:
        if not self.plans_ids:
            db = init_mongo()
            self.plans_ids = [p['_id'] for p in db.planes.find({'merchant_id': self.id}, {'_id': 1})]
        return self.plans_ids

    def get_application_fee(self, plan_id: ObjectId) -> Optional[float]:
        for p in self.planes_multinegocio:
            if p['plan_id'] == plan_id:
                return p['application_fee']

    def get_application_fee_by_cobro(self, cobro: str) -> float:
        default_application_fee = 0
        return self.application_fee.get(cobro.lower(), default_application_fee)

    def processors(self, marketplace: bool) -> list:
        processors = []
        if marketplace:
            if self.sdk_decidir_marketplace.ok():
                processors.append('decidir')
            if self.sdk_mercadopago_marketplace.ok():
                processors.append('mercadopago')
        else:
            if self.sdk_decidir.ok():
                processors.append('decidir')
            if self.sdk_mercadopago.ok():
                processors.append('mercadopago')
        return processors

    def get_sdk(self, processor: str, marketplace: bool = False) -> SDK:
        if processor == 'decidir':
            return self.sdk_decidir_marketplace if marketplace else self.sdk_decidir
        return self.sdk_mercadopago_marketplace if marketplace else self.sdk_mercadopago



from flask_restful import abort


def construct_pagination(results, url, page, limit, count):
    start = ((page - 1) * limit) + 1
    if count < start or limit < 0:
        abort(400, message="No hay resultados.")
    pagination = dict()
    pagination["data"] = results
    pagination["count"] = count
    pagination["start"] = start
    pagination["limit"] = limit
    if page == 1:
        pagination['previous'] = ''
    else:
        page_copy = page - 1
        limit_copy = start - 1
        pagination['previous'] = url + '?page=%d&limit=%d' % (page_copy, limit_copy)
    # make next url
    if start + limit > count:
        pagination['next'] = ''
    else:
        page_copy = page + 1
        pagination['next'] = url + '?page=%d&limit=%d' % (page_copy, limit)

    return pagination


def get_paginated_list(results, url, start, limit):
    start = int(start)
    limit = int(limit)
    count = len(results)
    if count < start or limit < 0:
        abort(400, message="No hay resultados.")
    # make response
    obj = {}
    obj['start'] = start
    obj['limit'] = limit
    obj['count'] = count
    # make URLs
    # make previous url
    if start == 1:
        obj['previous'] = ''
    else:
        start_copy = max(1, start - limit)
        limit_copy = start - 1
        obj['previous'] = url + '?start=%d&limit=%d' % (start_copy, limit_copy)
    # make next url
    if start + limit > count:
        obj['next'] = ''
    else:
        start_copy = start + limit
        obj['next'] = url + '?start=%d&limit=%d' % (start_copy, limit)
    # finally extract result according to bounds
    obj['results'] = results[(start - 1):(start - 1 + limit)]
    return obj



from datetime import datetime, date
import typing
import iso8601
import json
from bson import ObjectId
from .dates import localize


def is_jsonable(x):
    try:
        json.dumps(x)
        return True
    except (TypeError, OverflowError):
        return False


def datetime_parser(obj: typing.Union[dict, list, str]) -> typing.Union[dict, list, str]:
    """ Parsear las fechas de un diccionario a formato iso 8601

    :param obj: Diccionario, lista o string a parsear
    :type obj: Diccionario, lista o string
    :return: Diccionario, lista o string parseado
    :rtype: Diccionario, lista o string
    """
    if isinstance(obj, dict):
        for k, v in obj.items():
            if isinstance(v, dict) or isinstance(v, list):
                obj[k] = datetime_parser(v)
            if isinstance(v, str) and len(v) > 9:
                try:
                    obj[k] = iso8601.parse_date(v)
                except:
                    pass
            if isinstance(v, str) and len(v) == 24:
                try:
                    obj[k] = ObjectId(v)
                except:
                    pass
    if isinstance(obj, list):
        for i, v in enumerate(obj):
            if isinstance(v, dict) or isinstance(v, list):
                obj[i] = datetime_parser(v)
            if isinstance(v, str) and len(v) > 9:
                try:
                    obj[i] = iso8601.parse_date(v)
                except:
                    pass
            if isinstance(v, str) and len(v) == 24:
                try:
                    obj[i] = ObjectId(v)
                except:
                    pass
    return obj


def json_serial(dct: dict) -> dict:
    for k, v in dct.items():
        if isinstance(v, dict):
            dct[k] = json_serial(v)
        elif isinstance(v, (datetime, date)):
            dct[k] = localize(v).isoformat()
        elif isinstance(v, list):
            dct[k] = list_to_json(v)
        elif is_jsonable(v):
            dct[k] = v
        else:
            dct[k] = str(v)
    return dct


def list_to_json(data: list) -> list:
    result = list()
    for e in data:
        if isinstance(e, dict):
            result.append(json_serial(e))
        elif isinstance(e, list):
            result.append(list_to_json(e))
        elif isinstance(e, (datetime, date)):
            result.append(localize(e).isoformat())
        elif is_jsonable(e):
            result.append(e)
        else:
            result.append(str(e))
    return result


def format_ids(dct: dict) -> dict:
    """ Cambiar las claves de diccionario "_id" a "id"

    :param dct: diccionario con claves "_id"
    :type dct: dict
    :return: Diccionario modificado
    :rtype: dict
    """    
    r = dict()
    for k, v in dct.items():
        if k == "_id":
            r["id"] = v
        else:
            r[k] = v
    return r


def format_object_ids(dct: dict) -> dict:
    """ Crear un diccionario igual al que se recibe por parametro pero las \
        claves "id" se actualizan a "_id" y los valores de estas claves se \
        convierten a ObjectId

    :param dct: diccionario original
    :type dct: dict
    :return: diccionario actualizado
    :rtype: dict
    """    
    r = dict()
    for k, v in dct.items():
        if k == "id":
            r["_id"] = ObjectId(v)
        else:
            r[k] = v
    return r


def fix_bad_characters(string_to_fix:str)-> str:
    """ Intenta corregir errores de malos caracteres en un string
    :param string_to_fix: el string que se quiere corregir
    :type string_to_fix: str
    :return string_fixed: el string arreglado
    :rtype: str
    """
    try:
        string_fixed = string_to_fix.encode('iso-8859-1').decode('utf-8')
    except:
        string_fixed = string_to_fix # No pudo corregir el string entonces queda

    caracteres_aceptados = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ áéíóúÁÉÍÓÚäëöüÄËÖÜñÑ'
    for s in string_fixed:
        if not (s in caracteres_aceptados):
            string_fixed = string_fixed.replace(s, '')

    return string_fixed

def muvi_print(color_message:str,message:str) -> str:
    """ Colores para mensajes
    :param color_message: puede ser error,warning,info,successful
    :type color_message: str
    :param message: variable a printear
    :type message: any
    """
    color_message = color_message.lower()
    error = "\x1b[0;30;41m"
    warning = "\x1b[0;37;43m"
    info = "\x1b[0;30;44m"
    successful = "\x1b[0;30;42m"
    head = "\x1b[0;30;45m"
    normal = "\x1b[0m"

    if color_message=="error":
        print(error,message,normal)
    elif color_message=="warning":
        print(warning,message,normal)
    elif color_message=="info":
        print(info,message,normal)
    elif color_message=="successful":
        print(successful,message,normal)
    elif color_message=="head":
        print(head,message,normal)
    else:
        print(normal,message,normal)


import time
import email
import zeep.exceptions
import dateutil.parser
import xmltodict
import xml.etree.ElementTree as ET
from pprint import pprint
from cryptography import x509
from cryptography.hazmat.bindings.openssl.binding import Binding
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from dateutil.relativedelta import *
from .dates import today_argentina, localize
from .format import muvi_print
from .init_creds import test,init_mongo
import datetime
import os

TIME_TO_EXPIRATION = 12  # In hours


def make_xml(service):
    timestamp = int(time.time())
    unique_id = f"<uniqueId>{timestamp}</uniqueId>"
    created_at = today_argentina().isoformat(timespec="seconds")
    created_at = f"<generationTime>{created_at}</generationTime>"
    expires_at = (today_argentina() + relativedelta(hours=TIME_TO_EXPIRATION)
                  ).isoformat(timespec="seconds")
    expires_at = f"<expirationTime>{expires_at}</expirationTime>"
    header = f"{unique_id}{created_at}{expires_at}"
    body = f"<header>{header}</header><service>{service}</service>"
    print('Generation_time', created_at)
    print('Expiration_time:', expires_at)
    return f'<?xml version="1.0" encoding="UTF­8"?><loginTicketRequest version="1.0">{body}</loginTicketRequest>'


def sign_xml(tra, cert, private_key):
    if isinstance(tra, str):
        tra = tra.encode('utf8')

    if Binding:
        _lib = Binding.lib
        _ffi = Binding.ffi

        bio_in = _lib.BIO_new_mem_buf(tra, len(tra))
        if not private_key.startswith("-----BEGIN RSA PRIVATE KEY-----"):
            if isinstance(private_key, str):
                private_key = private_key.encode("utf-8")

        if isinstance(private_key, str):
            private_key = private_key.encode("utf-8")
        private_key = serialization.load_pem_private_key(
            private_key, None, default_backend()
        )

        if not cert.startswith("-----BEGIN CERTIFICATE-----"):
            if isinstance(cert, str):
                cert = cert.encode("utf-8")
        if isinstance(cert, str):
            cert = cert.encode("utf-8")
        cert = x509.load_pem_x509_certificate(cert, default_backend())

        try:
            # Firmar el texto (tra) usando cryptography (openssl bindings para python)
            p7 = _lib.PKCS7_sign(
                cert._x509, private_key._evp_pkey, _ffi.NULL, bio_in, 0
            )
        finally:
            # Liberar memoria asignada
            _lib.BIO_free(bio_in)
        # Se crea un buffer nuevo porque la firma lo consume
        bio_in = _lib.BIO_new_mem_buf(tra, len(tra))
        try:
            # Crear buffer de salida
            bio_out = _lib.BIO_new(_lib.BIO_s_mem())
            try:
                # Instanciar un SMIME
                _lib.SMIME_write_PKCS7(bio_out, p7, bio_in, 0)

                # Tomar datos para la salida
                result_buffer = _ffi.new("char**")
                buffer_length = _lib.BIO_get_mem_data(
                    bio_out, result_buffer)
                output = _ffi.buffer(result_buffer[0], buffer_length)[:]
            finally:
                _lib.BIO_free(bio_out)
        finally:
            _lib.BIO_free(bio_in)

        # Generar p7 en formato mail y recortar headers
        msg = email.message_from_string(output.decode("utf8"))
        for part in msg.walk():
            filename = part.get_filename()
            if filename == "smime.p7m":
                # Es la parte firmada?
                # Devolver CMS
                return part.get_payload(decode=False)


def analize_cert(crt, binary=False):
    "Carga un certificado digital y extrae los campos más importantes"

    if binary:
        cert = x509.load_pem_x509_certificate(crt, default_backend())
    else:
        if not crt.startswith("-----BEGIN CERTIFICATE-----"):
            crt = open(crt).read()
            if isinstance(crt, str):
                crt = crt.encode("utf-8")
        if isinstance(crt, str):
            crt = crt.encode("utf-8")
        cert = x509.load_pem_x509_certificate(crt, default_backend())
    certificate = dict()
    if cert:
        certificate = {
            'identidad': cert.subject,
            'caducidad': cert.not_valid_after,
            'emisor': cert.issuer,
            'certX509': cert
        }
    return certificate


def call_wsaa(client, cms, config, update_db=False):
    db = init_mongo()
    try:
        xml = client.service.loginCms(cms)
        xml = xml.replace('<?xml version="1.0" encoding="UTF8" standalone="yes"?>', '')
        xml = xml.replace('<?xml version="1.0" encoding="UTF8"?>', '')
        ticket = xmltodict.parse(xml)
        new_afip = {
            'token': ticket['loginTicketResponse']['credentials']['token'],
            'sign': ticket['loginTicketResponse']['credentials']['sign'],
            'expiration_token_date': dateutil.parser.parse(ticket['loginTicketResponse']['header']['expirationTime'])
        }
        if test:
            print('Guardando el backup...')
            now_str = datetime.datetime.now().strftime('%Y_%m_%d_%H_%M_%S')
            try:
                os.mkdir('backup_token')
            except:
                pass
            f = open('backup_token/token_backup_' + now_str + '.json', "w")
            f.write(str(new_afip))
            f.close()
        if update_db:
            print('Guardando en la base de datos...')
            r = db.configuraciones.update_one({'_id': config['_id']}, {'$set': new_afip})
            muvi_print('succesful', f"Se actualizo {r.modified_count} config")
        else:
            muvi_print('warning', 'WARNING! Se opto por no realizar un update en la base de datos')
        return new_afip
    except zeep.exceptions.Fault as e:
        print(f'Error: {e.code}: {e.message}')
        return


def authorize(client, config, service) -> dict:
    ticket = dict()
    if 'expiration_token_date' in config.keys():
        if localize(config['expiration_token_date']) > today_argentina():
            print('Token no vencido, se busca en la DB...')
            ticket['token'] = config['token']
            ticket['sign'] = config['sign']
            ticket['expiration_token_date'] = config['expiration_token_date']
            muvi_print('successful', 'TOKEN OBTENIDO DE LA DB CON EXITO')
            return ticket
        else:
            tra = make_xml(service)
            cms = sign_xml(tra=tra, cert=config['certificado'], private_key=config['private_key'])
            ticket = call_wsaa(client, cms, config, True)
            if ticket:
                muvi_print('successful', 'TOKEN CREADO CON EXITO')
            else:
                muvi_print('error', 'ERROR AL CREAR EL TOKEN')
            return ticket
    else:
        tra = make_xml(service)
        cms = sign_xml(tra=tra, cert=config['certificado'], private_key=config['private_key'])
        ticket = call_wsaa(client, cms, config, True)
        if ticket:
            muvi_print('successful', 'TOKEN CREADO CON EXITO')
        else:
            muvi_print('error', 'ERROR AL CREAR EL TOKEN')
        return ticket



from datetime import datetime, timedelta
from typing import Optional

from muvinai.utilities.dates import today_argentina, get_periodo, DIAS_ANTICIPO_DE_PAGO
from muvinai.utilities.payments import datetime_parser, get_client_price, get_access_amount
from muvinai.utilities.init_creds import init_mongo, test

from muvisdk import SDK

from .muviBase import MuviBase
from .merchant import Merchant
from .plan import Plan
from .card import Card


class Boleta(MuviBase):
    @classmethod
    def get(cls, boleta_id):
        db = init_mongo()
        boleta_db = db.boletas.find_one(boleta_id)
        return Boleta(boleta_db) if boleta_db else None

    def __init__(self, boleta_db: dict = None, cliente=None, merchant: Merchant = None, plan: Plan = None):
        super().__init__(boleta_db)
        if not boleta_db:
            boleta_db = {}
        self.id = boleta_db.get('_id')
        self.member_id = boleta_db.get('member_id')
        self.date_created = boleta_db.get('date_created')
        self.original_payment_date = boleta_db.get('original_payment_date')
        self.source = boleta_db.get('source')
        self.tries = boleta_db.get('tries')
        self.status = boleta_db.get('status')
        self.merchant_id = boleta_db.get('merchant_id')
        self.charges_detail = boleta_db.get('charges_detail')
        self.transaction_details = boleta_db.get('transaction_details')
        self.period = boleta_db.get('period')
        self.plan_id = boleta_db.get('plan_id')
        self.seller_merchant_id = boleta_db.get('seller_merchant_id')
        self.application_fee = boleta_db.get('application_fee')
        self.mail_dates = boleta_db.get('mail_dates')

        self.cliente = cliente
        self.plan = plan
        self.merchant = merchant if merchant and merchant.id == self.seller_merchant_id else None
        self.seller_merchant = merchant if merchant and merchant.id == self.seller_merchant_id else None

    def create(self, source: str, cliente, merchant: Merchant = None, plan: Plan = None):
        self.id = None
        self.cliente = cliente
        self.plan = plan
        self.source = source
        self.merchant = merchant if merchant and merchant.id == self.seller_merchant_id else None
        self.seller_merchant = merchant if merchant and merchant.id == self.seller_merchant_id else None
        self.status = "error_not_processed"
        charges_detail = self.calculate_price()

        if "checkout" in source or "cobro_aon" in source or "pending_accesos" == source:
            opd = today_argentina()
            self.period = get_periodo(opd)
        else:
            opd = cliente.next_payment_date
            self.period = get_periodo(opd + timedelta(days=DIAS_ANTICIPO_DE_PAGO))

        plan = self.get_plan()
        self.member_id = cliente.id
        self.date_created = today_argentina()
        self.original_payment_date = opd
        self.tries = []
        self.status = self.status
        self.charges_detail = charges_detail
        self.merchant_id = plan.merchant_id
        self.seller_merchant_id = None
        self.application_fee = None
        self.period = self.period
        self.plan_id = plan.id
        self.mail_dates = None
        self.update(
            member_id=self.member_id,
            date_created=self.date_created,
            original_payment_date=self.original_payment_date,
            source=self.source,
            tries=self.tries,
            status=self.status,
            merchant_id=self.merchant_id,
            charges_detail=self.charges_detail,
            period=self.period,
            plan_id=self.plan_id
        )

        self.seller_merchant = cliente.get_seller_merchant()
        if self.seller_merchant:
            if plan.es_sportclub_cadena:
                application_fee = self.seller_merchant.get_application_fee(plan.id)
                if application_fee is None:
                    self.update(status='error_application_fee')
                    return
            else:
                application_fee = plan.get_application_fee()
                if application_fee is None:
                    application_fee = self.seller_merchant.get_application_fee_by_cobro(plan.cobro)

            if application_fee > self.final_price:
                application_fee = self.final_price

            self.update(
                application_fee=application_fee,
                seller_merchant_id=self.seller_merchant.id
            )

    @property
    def final_price(self) -> Optional[float]:
        return self.charges_detail.get('final_price') if self.charges_detail else None

    @property
    def last_payment_id(self):
        return self.tries[-1]['payment_id'] if self.tries else None

    def set_status(self, status: str) -> None:
        if self.data["status"] != 'restored' or status == 'approved':
            self.update(status=status)

    def get_cliente(self):
        if not self.cliente:
            from .cliente import Cliente
            db = init_mongo()
            self.cliente = Cliente(db.clientes.find_one(self.member_id))
        return self.cliente

    def get_plan(self) -> Plan:
        if not self.plan:
            db = init_mongo()
            plan_db = db.planes.find_one(self.plan_id)
            self.plan = Plan(plan_db) if plan_db else None
        return self.plan

    def get_merchant(self) -> Merchant:
        if not self.merchant:
            db = init_mongo()
            merchant_db = db.merchants.find_one(self.merchant_id)
            self.merchant = Merchant(db.merchants.find_one(self.merchant_id)) if merchant_db else None
        return self.merchant

    def get_seller_merchant(self) -> Optional[Merchant]:
        if not self.seller_merchant_id:
            return None

        if not self.seller_merchant:
            db = init_mongo()
            seller_merchant_db = db.merchants.find_one(self.seller_merchant_id)
            self.seller_merchant = Merchant(seller_merchant_db) if seller_merchant_db else None
        return self.seller_merchant

    def get_date_last_mail(self) -> Optional[datetime]:
        if self.mail_dates:
            return self.mail_dates[-1]

    def register_mail_date(self) -> None:
        mail_dates = self.mail_dates if self.mail_dates else []
        mail_dates.append(today_argentina())
        self.update(mail_dates=mail_dates)

    def is_restored(self) -> bool:
        return self.status == 'restored'

    def get_last_status(self) -> str:
        return self.tries[-1]['status'] if self.tries else self.status

    def get_last_status_detail(self) -> Optional[str]:
        return self.tries[-1]['status_detail'] if self.tries else None

    def is_marketplace(self) -> bool:
        return self.application_fee is not None and 0 < self.application_fee < self.charges_detail['final_price']

    def calculate_price(self) -> dict:
        plan = self.get_plan()
        plan_price = plan.price
        cliente = self.get_cliente()
        corporativo = cliente.get_corporativo()
        corpo_discount = corporativo.porcentaje_descuento_empleado if corporativo else 0
        sum_access = plan.nivel_de_acceso == "Flex"
        access_count_limit = 6

        if self.source == 'pending_accesos':
            accesos = get_access_amount(cliente.id, max_count=access_count_limit)
            charges_detail = {'final_price': accesos, 'access': accesos}
        else:
            charges_detail = get_client_price(cliente.data, plan_price, corpo_discount,
                                              sum_access=sum_access, access_count_limit=access_count_limit)
            cliente.apply_discounts(plan_price)
        return charges_detail

    def agregar_transaction_details(self, payment: dict) -> None:
        if payment.get('transaction_details'):
            self.update(transaction_details=payment['transaction_details'])

    def agregar_intento(self, payment_result: dict, card: Card) -> None:
        print('     Agregando intento en el array de tries')
        processor = payment_result.get('processor')

        intento = {
            'try_number': len(self.data['tries']) + 1,
            'payment_day': today_argentina(),
            'payment_type': card.card_type,
            'card_brand': card.card_brand,
            'card_id': card.get_id(processor),
            'status': payment_result['status'],
            'status_detail': payment_result['status_detail'],
            'payment_id': payment_result.get('id', 400),
            'processor': processor
        }

        self.tries.append(intento)
        self.update(tries=self.tries)
        self.set_status(intento['status'])

    def agregar_intento_error(self, status_detail: str, processor: str = None, card: Card = Card.empty_card()) -> None:
        print(f"    ERROR: {status_detail}")
        r = {
            "status": "error",
            "status_detail": status_detail
        }
        if processor:
            r['processor'] = processor
        self.agregar_intento(r, card)

    def agregar_intento_precio_cero(self) -> None:
        status_detail = "Pago no realizado por ser precio 0."
        print(f"    {status_detail}")
        r = {
            "status": "approved",
            "status_detail": status_detail,
            "id": 11
        }
        self.agregar_intento(r, Card.empty_card())

    def agregar_intento_pending_efectivo(self) -> None:
        status_detail = "Pago pendiente a pagar efectivo"
        print(f"    {status_detail}")
        r = {
            "status": "pending_efectivo",
            "status_detail": status_detail,
            "id": 11
        }
        self.agregar_intento(r, Card.empty_card())

    def create_payment_data(self, card: Card, card_token: str) -> dict:
        plan = self.get_plan()
        cliente = self.get_cliente()

        n_try = len(self.data['tries']) + 1
        numero_intento = "Primer intento" if n_try == 1 else f"Reintento {n_try}"
        merchant = self.get_merchant()
        seller_merchant = self.get_seller_merchant() if self.seller_merchant_id else merchant
        brand = merchant.get_brand()

        description = f"{brand.name} - {plan.name} - {self.period}"
        if seller_merchant.activate_facturacion:
            description = "Suscripción SportClub"

        payment_data = {
            "additional_info": {
                "items": [{
                    "id": str(plan.id),
                    "title": f'{plan.name} - {cliente.nombre} {cliente.apellido} - {cliente.email} - {self.period}',
                    "category_id": plan.nivel_de_acceso,
                    "description": f'{numero_intento} - id: {cliente.id}'
                }],
                "shipments": {
                    "receiver_address": {
                        "state_name": cliente.domicilio.provincia,
                        "city_name": cliente.domicilio.localidad,
                        "street_name": cliente.domicilio.calle,
                        "street_number": cliente.domicilio.altura,
                    }
                },
                "payer": {
                    "first_name": cliente.nombre,
                    "last_name": cliente.apellido,
                    "address": None
                }
            },
            "notification_url": None if test else f"https://notificaciones.buen.club/notifications"
                                                  f"?source_news=webhooks&merchant={str(seller_merchant.id)}",
            "transaction_amount": round(self.data["charges_detail"]["final_price"], 2),
            "token": card_token,
            "description": description,
            "bin": card.first_six_digits,
            "installments": 1,  # ??
            "payer": cliente.data,
            "external_reference": f'{brand.name} - {self.source} - {cliente.documento}'
        }

        if card.payment_method_id:
            payment_data['payment_method_id'] = card.payment_method_id

        if 'application_fee' in self.data and 0 < self.data['application_fee'] < payment_data['transaction_amount']:
            payment_data['application_fee'] = round(self.data['application_fee'], 2)

        if self.status != 'restored':
            payment_data['point_of_interaction'] = self.calculate_poi()

        return payment_data

    def procesar_pago(self, sdk: SDK, sdk_cadena: SDK, card: Card) -> None:
        card_token = card.tokenize(sdk_cadena) if self.is_marketplace() else card.tokenize(sdk)
        if not card_token:
            self.agregar_intento_error(status_detail='Falló la creación de token de la tarjeta.',
                                       processor=sdk.processor,
                                       card=card)
            return

        print(f'        Token generado OK')

        payment_data = self.create_payment_data(card, card_token)

        if self.application_fee is not None and self.application_fee == self.final_price:
            payment_attempt = sdk_cadena.payment().create(payment_data)
        else:
            payment_attempt = sdk.payment().create(payment_data)

        payment_response = datetime_parser(payment_attempt['response'])
        processor = payment_attempt['response']['processor']

        print("         El estado del pago es: {}".format(payment_response["status"]))
        if "id" in payment_response:
            print("         El id del pago: {}".format(payment_response["id"]))

        # si hay algun error en el pago
        if payment_attempt["status"] >= 299:
            print(payment_response)
            status_detail = payment_response.get('message', 'Falló el intento de pago.')
            self.agregar_intento_error(status_detail=status_detail, processor=processor, card=card)
            return

        # si el pago es 200
        self.agregar_intento(payment_response, card)
        if self.status == 'approved':
            self.agregar_transaction_details(payment_response)

    def pagar(self) -> None:
        if not self.charges_detail:
            self.agregar_intento_error(status_detail='Error al calcular el precio del cliente.')
            return

        if self.charges_detail["final_price"] == 0:
            self.agregar_intento_precio_cero()
            return

        cliente = self.get_cliente()
        if cliente.get_preferred_payment_method() == "efectivo":
            print("    Boleta generada con status pending_efectivo")
            if len(self.tries) == 0 or self.tries[-1]['status'] != 'pending_efectivo':
                self.agregar_intento_pending_efectivo()
            return

        cadena = Merchant.get_cadena()
        marketplace = self.seller_merchant_id is not None
        merchant = self.get_seller_merchant() if marketplace else self.get_merchant()
        if not merchant.processors(marketplace):
            print("    El merchant no tiene ningun prcesador habilitado")
            self.agregar_intento_error(status_detail="no_payment_processor")
            return

        card = cliente.get_card()
        card_processors = cliente.get_card().processors()
        if not card_processors:
            self.agregar_intento_error(status_detail='Card not found', card=card)
            return

        print(f"    Se procede a realizar un pago de ${self.data['charges_detail']['final_price']}")
        print(f'    La tarjeta {card.id} tiene los procesadores: {card_processors}')
        for processor in card_processors:
            print(f'    processor: {processor} | marketplace: {marketplace}')
            sdk = merchant.get_sdk(processor=processor, marketplace=marketplace)
            sdk_cadena = cadena.get_sdk(processor=processor)
            if not sdk.ok():
                print(f'    No se pudo instanciar el SDK {processor} {marketplace}')
                continue

            if not card.get_id(processor):
                self.agregar_intento_error(status_detail='Card not found', card=card)
                continue

            print(f"    Se procede a realizar el pago con {processor}")
            self.procesar_pago(sdk, sdk_cadena, card)
            if self.get_last_status_detail() and ('Payment not found' in self.get_last_status_detail() or
                                                  'Invalid client' in self.get_last_status_detail()):
                print(f"    Hubo un error de tipo Payment not found. "
                      f"    Se borra el poi del cliente y se vuelve a procesar el pago")
                cliente.update(poi=None)
                self.procesar_pago(sdk, sdk_cadena, card)

            if self.get_last_status() in ["approved", "in_process"]:
                break

    def calculate_poi(self) -> dict:
        point_of_interaction = {
            "transaction_data": {}
        }

        cliente = self.get_cliente()
        poi = cliente.poi
        if poi:
            first_time_use = False
            number = poi["installments"] + 1
            subscription_sequence = str(poi["payment_reference"])
            point_of_interaction["transaction_data"]["payment_reference"] = {"id": subscription_sequence}
        else:
            first_time_use = True
            number = 1

        merchant = self.get_seller_merchant() if self.seller_merchant_id else self.get_merchant()

        point_of_interaction["transaction_data"].update({
            "first_time_use": first_time_use,
            "subscription_sequence": {
                "number": number,
                "total": 32
            },
            "invoice_period": {
                "period": number,
                "type": "monthly"
            },
            "billing_date": today_argentina().strftime("%Y-%m-%d"),
            "subscription_id": f"{merchant.application_id}-{cliente.documento}"
        })
        point_of_interaction["type"] = "SUBSCRIPTIONS"

        return point_of_interaction

    def push_to_db(self) -> None:
        db = init_mongo()
        if not self.id:
            r = db.boletas.insert_one(self._to_update)
            self.update(id=r.inserted_id)
            print("Boleta insertada en db", r.inserted_id)
        else:
            r = db.boletas.update_one({"_id": self.id}, {"$set": self._to_update})
            print("Boleta actualizada en db", r.matched_count, r.modified_count)



from muvinai.utilities.init_creds import init_mongo
from muvinai.utilities.dates import today_argentina, get_periodo, DIAS_ANTICIPO_DE_PAGO, DIAS_GRACIA
# from muvinai.utilities.members import member_unsubscribe
from muvinai.utilities.mail_sender import (send_mail_pago_en_efectivo, send_mail_cambio_tarjeta,
                                           send_mail_cambio_tarjeta_aon, send_mail_pago_pendiente)
from muvinai.utilities.payments import calculate_payment_date, set_next_vigency
from muvinai.utilities.sportaccess import crear_usuario_sportclub

from .muviBase import MuviBase
from .plan import Plan
from .corporativo import Corporativo
from .merchant import Merchant
from .boleta import Boleta
from .card import Card

from datetime import datetime, timedelta
from typing import Optional
from copy import deepcopy


class ClientesIterator:
    def __init__(self, clientes):
        self.clientes = clientes
        self.index = 0

    def __next__(self):
        return Cliente(self.clientes.next())


class Clientes:
    def __init__(self, query=None, lookup=None):
        db = init_mongo()

        if not query:
            query = {}

        pipeline = [
            {
                '$match': query
            }
        ]

        if lookup:
            for c in lookup:
                if c == 'planes':
                    pipeline.append({
                        '$lookup': {
                            'from': 'planes',
                            'localField': 'active_plan_id',
                            'foreignField': '_id',
                            'as': 'plan'
                        }
                    })
                if c == 'corporativo' and 'planes' in lookup:
                    pipeline.append({
                        '$lookup': {
                            'from': 'planes',
                            'localField': 'active_plan_id',
                            'foreignField': '_id',
                            'as': 'plan'
                        }
                    })

        self.clientes = db.clientes.aggregate(pipeline)

    def __iter__(self):
        return ClientesIterator(self.clientes)


class Cliente(MuviBase):
    @classmethod
    def search(cls, query=None):
        return Clientes(query)

    def __init__(self, cliente_db: dict, plan: Plan = None, corporativo: Corporativo = None):
        super().__init__(cliente_db)
        self.id = cliente_db['_id']
        self.documento = cliente_db['documento']
        self.nombre = cliente_db['nombre']
        self.apellido = cliente_db['apellido']
        self.status = cliente_db['status']
        self.email = cliente_db['email']
        self.domicilio = Domicilio(cliente_db['domicilio'])
        self.discounts = cliente_db['discounts'] if 'discounts' in cliente_db else None
        self.fecha_vigencia = cliente_db['fecha_vigencia']
        self.next_payment_date = cliente_db['next_payment_date']
        self.last_subscription_date = cliente_db['last_subscription_date']
        self.period_init_day = cliente_db['period_init_day']
        self.active_plan_id = cliente_db['active_plan_id']
        self.plan_corporativo = cliente_db.get('plan_corporativo')
        self.cards = cliente_db.get('cards', [])
        self.preferred_payment_method = cliente_db['preferred_payment_method']
        self.brand_name = cliente_db['brand_name']
        self.payment_ids = cliente_db.get('payment_ids', [])
        self.last_payment_id = cliente_db.get('last_payment_id')
        self.history = cliente_db.get('history', [])
        self.poi = cliente_db.get('poi')
        self.cobros_recurrentes = cliente_db.get('cobros_recurrentes', 0)
        self.seller_merchant_id = cliente_db.get('seller_merchant_id')

        self.plan = plan if plan and plan.id == cliente_db['active_plan_id'] else None
        self.corporativo = corporativo if corporativo and corporativo.id == cliente_db['plan_corporativo'] else None
        self.seller_merchant = None

    def get_plan(self) -> Plan:
        if not self.plan:
            db = init_mongo()
            self.plan = Plan(db.planes.find_one(self.active_plan_id))
        return self.plan

    def get_corporativo(self) -> Optional[Corporativo]:
        if not self.plan_corporativo:
            return None

        if self.corporativo:
            db = init_mongo()
            self.corporativo = Corporativo(db.corporativo.find_one(self.plan_corporativo))
        return self.corporativo

    def get_seller_merchant(self) -> Optional[Merchant]:
        if not self.seller_merchant_id:
            return None

        if not self.seller_merchant and self.plan and self.seller_merchant_id == self.plan.merchant_id:
            self.seller_merchant = self.plan.get_merchant()

        if not self.seller_merchant:
            db = init_mongo()
            self.seller_merchant = Merchant(db.merchants.find_one(self.seller_merchant_id))
        return self.seller_merchant

    def existe_boleta_para_periodo_corriente(self) -> bool:
        db = init_mongo()
        existing_period = db.boletas.find_one({
            "member_id": self.id,
            "period": get_periodo(self.next_payment_date + timedelta(days=DIAS_ANTICIPO_DE_PAGO)),
            "source": {
                "$regex": "recurring"
            },
            "status": {
                "$nin": ["refunded", "expired"]
            }
        })
        if not existing_period:
            return False  # crea la boleta nueva

        boleta_checkout = db.boletas.find_one({
            "source": {
                "$regex": "checkout"
            },
            "original_payment_date": {
                "$gt": existing_period["original_payment_date"]
            }
        })
        return boleta_checkout is None  # devuelve true si no existe la de checkout --> no la crea

    def crear_boleta(self, source: str) -> Boleta:
        boleta = Boleta()
        boleta.create(source, cliente=self, merchant=self.get_plan().get_merchant(), plan=self.get_plan())
        return boleta

    # def member_unsubscribe(self, reason: str, source: str):
    #     return member_unsubscribe(self.data, reason, source)

    def get_preferred_payment_method(self) -> str:
        plan = self.get_plan()
        if plan.payment_methods[self.preferred_payment_method]:
            # el plan acepta el ppm del cliente
            return self.preferred_payment_method

        if plan.payment_methods["tarjeta"] or not plan.payment_methods["efectivo"]:
            # el plan no acepta ningun metodo de pago, se usa tarjeta
            return "tarjeta"

        return "tarjeta" if plan.payment_methods["tarjeta"] else "efectivo"

    def get_card(self) -> Card:
        for card in self.cards:
            if card['valid']:
                return Card(card)
        return Card()

    def send_mail_nuevo_pago_en_efectivo(self, boleta: Boleta) -> None:
        receiver = {'nombre': self.nombre, 'email': self.email}
        plan = boleta.get_plan()
        merchant = boleta.get_merchant()
        brand = merchant.get_brand()
        send_mail_pago_en_efectivo(receiver, brand.brand_db, boleta.final_price, plan.name)

    def send_mail_pago_en_efectivo_pendiente(self, boleta: Boleta) -> None:
        receiver = {'nombre': self.nombre, 'email': self.email}
        merchant = boleta.get_merchant()
        brand = merchant.get_brand()
        send_mail_pago_pendiente(receiver, brand.brand_db)

    def send_mail_cambio_de_tarjeta(self) -> dict:
        merchant = self.get_plan().get_merchant()
        brand = merchant.get_brand()
        seller_merchant = self.get_seller_merchant()
        sede = seller_merchant.get_sede() if seller_merchant else merchant.get_sede()
        if self.brand_name.lower() == 'sportclub':
            return send_mail_cambio_tarjeta(self.data, brand.brand_db, sede.email_contacto)
        elif self.brand_name.lower() == 'aon':
            return send_mail_cambio_tarjeta_aon(self.data, brand.brand_db)

    def apply_discounts(self, plan_price: float) -> dict:
        corporativo = self.get_corporativo()
        corpo_discount = corporativo.porcentaje_descuento_empleado if corporativo else 0
        discounts = {}
        if corpo_discount > 0:
            discounts["corpo_discount"] = round(- plan_price * corpo_discount / 100, 2)

        if self.discounts:
            try:  # TODO: Agregar validations para sacar este try
                for discount in self.discounts:
                    if discount['aplicaciones_restantes'] > 0:
                        concepto = discount['concepto'] if 'concepto' in discount else 'n/a'
                        discount_price = plan_price * discount["porcentaje"] / 100 + discount["monto_absoluto"]
                        discounts[concepto] = round(- discount_price, 2)
                        discount['aplicaciones_restantes'] -= 1
            except:
                print(f"    ERROR en el cálculo de descuentos - se retorna un diccionario de precios vacío.")
                return {}

        self.update(discounts=self.discounts)

        return discounts

    def add_history_event(self, event: str, source: str = None) -> None:
        plan = self.get_plan()
        history = self.history if self.history else []
        history.append({
            'event': event,
            'date_created': today_argentina(),
            'source': source,
            'plan': plan.id
        })
        self.update(history=history)

    def crear_usuario_sportclub(self) -> None:
        plan = self.get_plan()
        try:
            crear_usuario_sportclub(self.data, self.payment_ids[-1], plan.nivel_de_acceso)
        except:
            print('Error al intentar crear_usuario_sportclub')

    def calculate_next_payment_date(self, boleta: Boleta) -> datetime:
        plan = self.get_plan()
        npd = calculate_payment_date(self.period_init_day, plan.cobro, boleta.period)
        print("         El próximo dia de cobro es el {}".format(npd.strftime("%d/%m/%y")))
        return npd

    def calculate_vigencia(self, npd: datetime, cobro: str) -> datetime:
        fecha_vigencia = set_next_vigency(npd, cobro)
        if self.status != 'activo' and cobro == 'Mensual':
            fecha_vigencia = fecha_vigencia - timedelta(DIAS_GRACIA)
        print("         Se actualiza la fecha de vigencia: {}".format(fecha_vigencia))
        return fecha_vigencia

    def update_poi(self, boleta: Boleta) -> None:
        if boleta.tries[-1]['processor'] != 'mercadopago':
            return

        poi = {
            "installments": self.poi["installments"] + 1 if self.poi else 1,
            "payment_reference": boleta.last_payment_id
        }

        self.update(poi=poi)

    def update_client(self, boleta: Boleta, update_npd: bool = False) -> None:
        self.update(lastModified=today_argentina())

        if boleta.last_payment_id:
            print("         Se inserta payment id en el cliente.")
            payment_ids = self.payment_ids
            payment_ids.insert(0, boleta.last_payment_id)
            self.update(payment_ids=payment_ids, last_payment_id=boleta.last_payment_id)

        npd = self.calculate_next_payment_date(boleta)
        if update_npd:
            self.update(next_payment_date=npd)

        plan = boleta.get_plan()
        if boleta.status == 'approved':
            self.update(
                fecha_vigencia=self.calculate_vigencia(npd, plan.cobro),
                cobros_recurrentes=self.cobros_recurrentes + 1
            )
            self.update_poi(boleta)

    def update_plan_herencia(self) -> bool:
        plan = self.get_plan()
        if plan.plan_herencia:
            self.add_history_event('plan_herencia', source='cobros_recurrentes')
            self.update(active_plan_id=plan.plan_herencia)
            self.plan = plan.get_plan_herencia()
            print("[Plan Herencia] El cliente {} con plan {} pasa a tener el plan {}".format(self.id,
                                                                                             self.active_plan_id,
                                                                                             plan.plan_herencia))
            return True
        return False

    def push_to_db(self) -> None:
        db = init_mongo()
        to_update = deepcopy(self._to_update)
        to_unset = {}
        if 'poi' in to_update and not to_update['poi']:
            to_unset['poi'] = ''
            del to_update['poi']

        r = db.clientes.update_one(
            {'_id': self.id},
            {
                '$set': to_update,
                '$unset': to_unset
            }
        )
        print('Cliente actualizado en db', r.matched_count, r.modified_count)


class Domicilio:
    def __init__(self, data):
        self.provincia = data['provincia'] if 'provincia' in data else None
        self.localidad = data['localidad'] if 'localidad' in data else None
        self.calle = data['calle'] if 'calle' in data else None
        self.altura = data['altura'] if 'altura' in data else None

        try:
            self.altura = int(self.altura)
        except ValueError:
            pass



from muvinai.utilities.init_creds import init_mongo
from .merchant import Merchant


class Plan:
    def __init__(self, plan_db, merchant=None):
        self.plan_db = plan_db
        self.id = plan_db['_id']
        self.name = plan_db['name']
        self.plan_herencia = plan_db.get('plan_herencia')
        self.plan_herencia_db = None
        self.cobro = plan_db['cobro']
        self.nivel_de_acceso = plan_db['nivel_de_acceso']
        self.price = plan_db['price']
        self.merchant_id = plan_db['merchant_id']
        self.payment_methods = plan_db['payment_methods']
        self.merchant = merchant
        self.application_fee = plan_db.get('application_fee')
        self.es_sportclub_cadena = plan_db.get('es-sportclub-cadena', False)

    def get_merchant(self) -> Merchant:
        if not self.merchant:
            db = init_mongo()
            self.merchant = Merchant(db.merchants.find_one(self.merchant_id))
        return self.merchant

    def get_plan_herencia(self):
        if not self.plan_herencia:
            return None
        if not self.plan_herencia_db:
            db = init_mongo()
            self.plan_herencia_db = Plan(db.planes.find_one(self.plan_herencia))
        return self.plan_herencia_db

    def get_application_fee(self) -> float:
        return self.application_fee



from bson import ObjectId
from dateutil.relativedelta import relativedelta

from ..utilities.dates import (
    calculate_payment_date,
    get_periodo,
    set_next_vigency,
    today_argentina,
)
from ..utilities.members import create_history_event


class Client:
    def __init__(self):
        pass

    def create(
        self,
        active_plan_id,
        payment_id,
        card=None,
        discount=None,
        plan_corporativo=None,
        **kwargs
    ):
        self.active_plan_id = ObjectId(active_plan_id)
        self.payment_ids = [payment_id]
        self.last_payment_id = payment_id
        self.plan_corporativo = plan_corporativo
        self.nacimiento = "22/02/2022"
        self.preferred_payment_method = "tarjeta"
        self.status = "pago en proceso"
        self.cobros_recurrentes = 0

        self.__dict__.update(kwargs)
        self.discounts = [] if not discount else [discount]
        self.cards = [] if not card else [card]

        if card:
            self.active_card = card["id"]

        today = today_argentina()
        f_vigencia_apto = today + relativedelta(days=30)
        self.apto_medico = {
            "url": "",
            "status": "validado",
            "fecha_vigencia": f_vigencia_apto.replace(
                day=f_vigencia_apto.day, hour=23, minute=59, second=59
            ),
        }

        self.history = [
            create_history_event(
                member=self.__dict__, event_type="alta", source="checkout"
            )
        ]

    def set_dates(self, cobro, date=None):
        if not date:
            date = today_argentina()

        npd = calculate_payment_date(date.day, cobro, get_periodo(date))
        self.next_payment_date = npd
        self.fecha_vigencia = (
            None
            if self.status == "pago en proceso"
            else set_next_vigency(npd, cobro=cobro)
        )
        self.last_subscription_date = None if self.status == "pago en proceso" else date
        self.period_init_day = date.day

    def import_from_db(self, client: dict):
        for key in client.keys():
            setattr(self, key, client[key])

    def push_to_db(self, db):
        client = db.clientes.find_one(
            {"documento": self.documento, "brand_name": self.brand_name}
        )
        if not client:
            inserted = db.clientes.insert_one(self.__dict__)
            self._id = inserted.inserted_id
        else:
            db.clientes.update_one(
                {"documento": self.documento, "brand_name": self.brand_name},
                {"$set": self.__dict__},
            )

    def to_dict(self):
        return self.__dict__



# Crea o modifica usuarios en sportaccess
import requests
import json
from .init_creds import init_mongo, test
from datetime import datetime
from pymongo import ReturnDocument
from .format import fix_bad_characters
from muvinai.utilities.dates import today_argentina, arg_tz

db = init_mongo()

zonas = {
    "Elite": "-1",
    "Total": "1",
    "Plus": "2",
    "Local": "7"
}

sedes = {
    "CORP": "1",
    "ECOM": "900",
    "ALIA": "901",
    "WORK": "1010",
    "MUVI": "900"
}

all_clubes = db.club.find({"sportaccess_id": {"$exists": True}})


# sede_map = {c["sportaccess_id"]: c["name"] for c in all_clubes}
# additional_map = {
#    1: "Corporate",
#    900: "e-Commerce",
#    901: "Alianzas"
# }
# sede_map.update(additional_map)


def crear_usuario_sportclub(cliente: dict, payment_id: int = -1, nivel_de_acceso: str = 'buscar_plan') -> dict:
    """ Crear un usuario en la base de datos de Sportclub

    :param cliente: cliente de mongodb
    :type cliente: dict
    :param payment_id: id del pago de mercadopago
    :type payment_id: str
    :param nivel_de_acceso: nivel de acceso del plan
    :type nivel_de_acceso: str
    :return: los valores posibles son: Elite, Total, Plus, Local, Flex
    :rtype: dict
    """
    db = init_mongo()
    url = "https://apisami.sportclubaccess.com/2.0/socio?"

    plan = db.planes.find_one({"_id": cliente["active_plan_id"]})
    if nivel_de_acceso == 'buscar_plan':
        try:
            nivel_de_acceso = plan['nivel_de_acceso']
        except:
            nivel_de_acceso = 'Total'
    try:
        if plan["sede_local"] is None:
            sede_value = sedes[cliente["sportaccess_id"][0:4]]
        else:
            club = db.club.find_one({"_id": plan["sede_local"]})
            sede_value = club["sportaccess_id"]
    except:
        sede_value = "1"

    if payment_id == -1:
        try:
            payment_id = cliente['payment_ids'][-1]
        except:
            payment_id = 11

    for a in cliente["domicilio"]:
        if not a:
            cliente["domicilio"][a] = "n/a"

    if "nacimiento" not in cliente.keys():
        cliente["nacimiento"] = "22/02/2022"
    elif not cliente["nacimiento"] or cliente["nacimiento"] == "n/a":
        cliente["nacimiento"] = "22/02/2022"

    if "celular" not in cliente.keys():
        cliente["celular"] = "n/a"
    elif not cliente["celular"]:
        cliente["celular"] = "n/a"

    parameters = {
        "username": "3rdPty2902-8891-20",
        "password": "HWXSroa76FSI226JeEtOd9SqDoDXCkGtJyXTk2qEAN2HI34bOVe",
        "nro_socio": cliente["sportaccess_id"],
        "nombre": cliente["nombre"],
        "apellido": cliente["apellido"],
        "documento": cliente["documento"],
        "email": cliente["email"],
        "sexo": "X",
        "nacimiento": cliente["nacimiento"].replace("/", "-"),
        "celular": cliente["celular"],
        "calle": cliente["domicilio"]["calle"] if "calle" in cliente["domicilio"] else 'n/a',
        "numero": cliente["domicilio"]["altura"] if "altura" in cliente["domicilio"] else 'n/a',
        "localidad": cliente["domicilio"]["localidad"] if "localidad" in cliente["domicilio"] else 'n/a',
        "cp": cliente["domicilio"]["código postal"] if "código postal" in cliente["domicilio"] else 'n/a',
        "ciudad": cliente["domicilio"]["localidad"] if "localidad" in cliente["domicilio"] else 'n/a',
        "provincia": cliente["domicilio"]["provincia"] if "provincia" in cliente["domicilio"] else 'n/a',
        "nacionalidad": "Argentino",
        "sede": sede_value,
        "zona": "7" if nivel_de_acceso == "Flex" else zonas[nivel_de_acceso],
        "mp_payment_id": int(payment_id),
        "vigencia": cliente["fecha_vigencia"].strftime("%d-%m-%Y")

    }

    lista = [key + "=" + str(parameters[key]) for key in parameters.keys()]
    querystrings = "&".join(lista)
    if test:
        print("Se saltea el post a sportaccess por ser test.")
        return {"status": True, "message": "No se llamó al servicio por ser TEST."}
    r = requests.post(url + querystrings)
    response = r.json()
    if not response["status"]:
        print("Falló la creación de usuario en bd sporclub")
        print(response["message"])
    else:
        print("Usuario creado satisfactoriamente en bd sportclub")

    return response


def consulta_socio(documento: str) -> dict:
    """ Obtener un socio de la base de datos de sportclub
great
    :param documento: numero de documento del socio que buscado
    :type documento: str
    :return: respuesta de SpAccess (puede ser un socio o un mensaje de error)
    :rtype: dict
    """
    url = "https://apisami.sportclubaccess.com/2.0/members?Documento="
    r = requests.get(url + documento)
    r = r.json()
    try:  # Intento corregir los malos caracteres que puedan tener
        r['data']['Apellido'] = fix_bad_characters(r['data']['Apellido'])
        r['data']['Nombre'] = fix_bad_characters(r['data']['Nombre'])
        r['data']['Direccion'] = fix_bad_characters(r['data']['Direccion'])
    except:
        pass
    # Intenta pasar a datetime la fecha
    try:
        r['data']['Vigencia'] = datetime.strptime(r['data']['Vigencia'], '%d/%m/%Y')
        r['data']["FechaIngreso"] = datetime.strptime(r['data']["FechaIngreso"], '%d/%m/%Y')
    except:
        pass
    # Intenta convertir el apto medico
    try:
        r['data']['AptoMedico'] = datetime.strptime(r['data']['AptoMedico'], '%d/%m/%Y')
    except:
        pass
    return r


def consulta_socio_x_nro_socio(nro_socio: str) -> dict:
    """ Obtener un socio de la base de datos de sportclub

    :param documento: numero de documento del socio que buscado
    :type documento: str
    :return: respuesta de SpAccess (puede ser un socio o un mensaje de error)
    :rtype: dict
    """
    url = "https://apisami.sportclubaccess.com/2.0/members?NroSocio="
    r = requests.get(url + nro_socio)
    r = r.json()
    try:  # Intento corregir los malos caracteres que puedan tener
        r['data']['Apellido'] = fix_bad_characters(r['data']['Apellido'])
        r['data']['Nombre'] = fix_bad_characters(r['data']['Nombre'])
        r['data']['Direccion'] = fix_bad_characters(r['data']['Direccion'])
    except:
        pass
    try:  # Intenta pasar a datetime la fecha
        r['data']['Vigencia'] = datetime.strptime(r['data']['Vigencia'], '%d/%m/%Y')
        r['data']["FechaIngreso"] = datetime.strptime(r['data']["FechaIngreso"], '%d/%m/%Y')
    except:
        pass
    # Intenta convertir el apto medico
    try:
        r['data']['AptoMedico'] = datetime.strptime(r['data']['AptoMedico'], '%d/%m/%Y')
    except:
        pass
    return r


def create_external_user(data: dict) -> dict:
    """ Crear un cliente externo en mongodb

    :param data: datos del socio (estos datos se obtienen de la base de sportclub)
    :type data: dict
    :return: datos del socio con los que se guardo en mongodb
    :rtype: dict
    """

    inv_zonas = {a: b for b, a in zonas.items()}
    na = "Elite" if data["Documento"] in strelites else inv_zonas[str(data["Zona"])]

    nuevo_socio = {
        "sportaccess_id": data["NroSocio"],
        "nivel_de_acceso": na,
        "nombre": data["Nombre"],
        "apellido": data["Apellido"],
        "documento": data["Documento"],
        "email": data["Mail"],
        "domicilio": {
            "calle": data["Direccion"],
            "altura": data["Numero"],
            "apto_lote": data["Piso"],
            "localidad": data["Localidad"],
            "provincia": data["Provincia"],
            "código postal": data["CPostal"]
        },
        "celular": data["Movil"],
        "status": "externo",
        "fecha_vigencia": data["Vigencia"],
        "nacimiento": data["Nacimiento"],
        "last_subscription_date": data["FechaIngreso"],
        "sede": data["Sede"],
        "sede_sqlid": int(data["Sede-SqlId"]),
        "plan": data["Plan"],
        "active_plan_id": None,
        "apto_medico_externo": data["AptoMedico"],
        "brand_name": 'SportClub'
    }
    socio = db.clientes.update_one({"documento": data["Documento"], "sportaccess_id": {"$exists": True}},
                                   {"$set": nuevo_socio}, upsert=True)
    nuevo_socio["_id"] = socio.upserted_id
    return nuevo_socio


def cambio_de_plan(documento: str):
    """ Actualizar en mongodb y en SpAccess el plan del socio

    :param documento: numero de documento del socio al cual se le quiere cambiar el plan
    :type documento: str
    """
    db = init_mongo()
    cliente = db.clientes.find_one({"documento": documento})
    cliente["active_plan"] = "total-mensual"
    cliente["nivel_de_acceso"] = "Total"
    crear_usuario_sportclub(cliente, cliente["last_payment_id"], cliente["nivel_de_acceso"])
    db.clientes.update_one({"documento": documento}, {"$set": cliente})


def update_external_user(data: dict):
    """ Actualizar cliente externo

    :param data: datos que se van a actualizar (tienen que estar las keys: Documento, \
                    Zona, NroSocio, Nombre, Apellido, Documento, Mail, Direccion, Numero, Piso, Localidad,\
                    Provincia, CPostal, Movil, Vigencia, Nacimiento, FechaIngreso)
    :type data: dict
    """
    inv_zonas = {a: b for b, a in zonas.items()}
    na = "Elite" if data["Documento"] in strelites else inv_zonas[str(data["Zona"])]

    socio = {
        "sportaccess_id": data["NroSocio"],
        "nivel_de_acceso": na,
        "nombre": data["Nombre"],
        "apellido": data["Apellido"],
        "documento": data["Documento"],
        "email": data["Mail"],
        "domicilio": {
            "calle": data["Direccion"],
            "altura": data["Numero"],
            "apto_lote": data["Piso"],
            "localidad": data["Localidad"],
            "provincia": data["Provincia"],
            "código postal": data["CPostal"]
        },
        "celular": data["Movil"],
        "status": "externo",
        "fecha_vigencia": data["Vigencia"],
        "nacimiento": data["Nacimiento"],
        "last_subscription_date": data["FechaIngreso"],
        "sede": data["Sede"],
        "sede_sqlid": int(data["Sede-SqlId"]),
        "plan": data["Plan"],
        "active_plan_id": None,
        "apto_medico_externo": data["AptoMedico"],
        "brand_name": 'SportClub'
    }
    return db.clientes.find_one_and_update({"documento": data["Documento"], "brand_name": "SportClub"}, {"$set": socio},
                                           return_document=ReturnDocument.AFTER)


def resolve_external_status(documento):
    _client = db.clientes.find_one({"documento": documento, "brand_name": "SportClub", "email": {"$not": {"$regex": "@migracion"}}})
    if not _client or _client["status"] == "inactivo":
        r = consulta_socio(documento)
        try:
            if r["code"] == 404:
                return "*El DNI ingresado no se encuentra en nuestros registros. " \
                       "Por favor ingresá un DNI válido para continuar."
            elif r["code"] > 299:
                return r["message"]
            else:
                return create_external_user(r["data"])
        except:
            return "El servicio de SportAccess no retorna información válida para este documento."
    elif _client["status"] == "externo":
        r = consulta_socio(documento)
        try:
            if r["code"] == 404:
                return "*El DNI ingresado no se encuentra en nuestros registros. " \
                       "Por favor ingresá un DNI válido para continuar."
            elif r["code"] > 299:
                return r["message"]
            else:
                return update_external_user(r["data"])
        except:
            return "El servicio de SportAccess no retorna información válida para este documento."

    else:
        return _client


def consulta_socio_proclub(documento: str) -> dict:
    """ Obtener un socio de la base de datos de sportclub
great
    :param documento: numero de documento del socio que buscado
    :type documento: str
    :return: respuesta de ProClub (puede ser un socio o None)
    :rtype: dict
    """
    url = "https://apisocios.sportclub.com.ar/members?Documento="
    headers = {
        "x-api-key": "VHGFDn_2212FFF**/-FFFF",
        "Content-Type": "application/json"
    }
    try:
        r = requests.get(url + documento, headers=headers)
        r = r.json()
    except Exception as e:
        print('Se atrapo una excepcion llamando a proclub. ERROR:', str(e))
        return dict()

    try:
        int(r["code"])
    except ValueError:
        print('Error en consulta_socio_proclub. Status code invalido. Respuesta:', r)
        return dict()
    if r["code"] >= 400:
        print('Error en consulta_socio_proclub. status_code =', r["code"], 'Respuesta:', r)
        return dict()
    if not r["data"]:
        print('Error en consulta_socio_proclub. No hay data. status_code =', r["code"], 'Respuesta:', r)
        return dict()

    proclub = r["data"][0]
    if "CredencialSpice" in proclub.keys():
        spice = proclub["CredencialSpice"]
    else:
        spice = None

    sede_socio = db.club.find_one({"sportaccess_id": int(proclub["Sede-SqlId"])})
    if sede_socio:
        sede_id = sede_socio["_id"]
        sede_name = sede_socio["name"]
    else:
        sede_id = "n/a"
        sede_name = "n/a"

    na = proclub["nivel_acceso"].lower()
    fv = datetime.strptime(proclub['Vigencia'], '%d/%m/%Y').replace(tzinfo=arg_tz, hour=15)
    status = "activo" if fv > today_argentina() else "inactivo"

    deuda_promocion, deuda_arancel = 0, 0

    if "QntDeudaPromocion" in proclub.keys():
        deuda_promocion = proclub["QntDeudaPromocion"]
    if "QntDeudaArancel" in proclub.keys():
        deuda_arancel = proclub["QntDeudaArancel"]

    # Intentamos conversión a formato perfilsocio
    socio = {
        "_id": None,
        "status": status,
        "fecha_vigencia": fv,
        "last_subscription_date": datetime.strptime(proclub["FechaIngreso"], '%d/%m/%Y').replace(tzinfo=arg_tz, hour=15),
        "apto_medico": {
            "fecha_vigencia": datetime.strptime(proclub['AptoMedico'], '%d/%m/%Y').replace(tzinfo=arg_tz, hour=15)
        },
        "apellido": proclub["Apellido"],
        "documento": documento,
        "nombre": proclub["Nombre"],
        "email": proclub["Mail"],
        "plan_details": {
            "nivel_de_acceso": na,
            "name": proclub["Plan"],
            "merchant_id": None,
            "sede_local_name": sede_name if na == "local" else None,
            "sede_local": sede_id if na == "local" else None,
            "convenio": proclub["Convenio"],
            "vertical": "Corporativo" if proclub["Convenio"] else "otros"
        },
        "token_spice": spice,
        "seller_merchant": None,
        "externo": True,
        "terceros": proclub["Terceros"],
        "telefono": proclub.get("Telefono"),
        "sede_sqlid": proclub["Sede-SqlId"],
        "sede_socio_name": sede_name,
        "sede_socio": sede_id,
        "proclub_id": proclub["NroSocio"],
        "deuda_promocion": deuda_promocion,
        "deuda_arancel": deuda_arancel
    }

    return socio


def update_spice(nro_socio: str, spice) -> dict:
    """ Obtener un socio de la base de datos de sportclub
great
    :param documento: numero de documento del socio que buscado
    :type documento: str
    :return: respuesta de SpAccess (puede ser un socio o un mensaje de error)
    :rtype: dict
    """
    url = "https://apisocios.sportclub.com.ar/members?NroSocio="
    headers = {
        "x-api-key": "VHGFDn_2212FFF**/-FFFF",
        "Content-Type": "application/json"
    }
    body = {"Credencial_spice": spice}
    r = requests.put(url + nro_socio, headers=headers, data=json.dumps(body))
    return r


strelites = [
"17968603",
"25647644",
"18110621",
"44160654",
"28508245",
"20540023",
"29571882",
"17331567",
"24690984",
"29695864",
"32215977",
"16492982",
"17083952",
"37050158",
"94824571",
"14915216",
"14951519",
"39644268",
"27923183",
"42119189",
"20340280",
"11726199",
"28417781",
"26687480",
"24612588",
"25540482",
"28281063",
"12731772",
"16356582",
"22344506",
"17513343",
"28985800",
"22157614",
"17004744",
"30409905",
"17800734",
"16495297",
"30211271",
"32451066",
"22878693",
"21138738",
"20420715",
"20051366",
"21495716",
"12549464",
"21951202",
"44642281",
"37806138",
"28325919",
"21842834",
"10313318",
"94093103",
"42930040",
"43034077"]



class Corporativo:
    def __init__(self, corporativo_db):
        self.corporativo_db = corporativo_db
        self.id = corporativo_db['_id']
        self.porcentaje_descuento_empleado = corporativo_db['porcentaje-descuento-empleado']



import datetime
from pprint import pprint
import bson

from .init_creds import init_mongo, test

import mercadopago
import requests

db = init_mongo()


def get_payment(payment_id, sdk: mercadopago.sdk):
    response = sdk.payment().search(filters={"id": payment_id})
    return response["response"]


def get_merchant_from_payment(payment_id: str):
    """ Obtener el merchant a partir de un pago de mercadopago

          :param payment_id: id del pago
          :type payment_id: str
          :return: el merchant del pago
          :rtype: ObjectId
          """
    for merchant in db.merchants.find({"credentials.mercadopago.access_token": {"$exists": True}}):
        try:
            sdk = mercadopago.SDK(merchant["credentials"]["mercadopago"]["access_token"])
        except KeyError:
            continue
        payment = get_payment(payment_id, sdk)
        if "results" in payment.keys():
            if payment["results"]:
                return merchant["_id"]


def get_sdk_from_payment(payment_id: str):
    """ Obtener el sdk a partir de un pago de mercadopago

      :param payment_id: id del pago
      :type payment_id: str
      :return: el sdk ya instanciado de mercadopago
      :rtype: mercadopago.SDK
      """
    for merchant in db.merchants.find({"credentials.mercadopago.access_token": {"$exists": True}}):
        try:
            sdk = mercadopago.SDK(merchant["credentials"]["mercadopago"]["access_token"])
        except KeyError:
            continue
        payment = get_payment(payment_id, sdk)
        if "results" in payment.keys():
            if payment["results"]:
                return sdk


def get_payments_from_user_id(mp_id: str, sdk: mercadopago.sdk, days=30, limit=10):
    """ Obtener todos los pagos en los últimos <days> días para un usuario dado un mercadopago_id

          :param mp_id: id del usuario en mercadopago. Sólo los números antes de guión
          :type payment_id: str
          :param sdk: El SDK de mercadopago correspondiente al plan del usuario
          :type sdk: SDK.mercadopago
          :param mp_id: id del usuario en mercadopago. Sólo los números antes de guión
          :type days: int
          :param limit: límite de resultados a obtener
          :type limit: int
          :return: lista de pagos que satisfacen query
          :rtype: list
    """

    if "-" in mp_id:
        mp_id, x = mp_id.split("-")

    payment_info = {"begin_date": f"NOW-{days}DAYS",
                 "end_date": "NOW",
                 "range": "date_created",
                 "sort": "date_created",
                 "limit": limit,
                 "offset": 0,
                 "payer.id": mp_id
                 }

    payments = sdk.payment().search(filters=payment_info)

    return payments["response"]["results"]


def get_sdk_from_user(user_id: bson.ObjectId):
    """ Obtener el SDK de mercadopago a partir de un id de usuario

           :param user_id: id del usuario en mongoDB.
           :type payment_id: bson.ObjectId
           :return: sdk de mercadopago
           :rtype: mercadopago.SDK
           """
    client = db.clientes.find_one({"_id": user_id})
    plan = db.planes.find_one({"_id": client["active_plan_id"]})
    merchant = db.merchants.find_one({"_id": plan["merchant_id"]})
    sdk = mercadopago.SDK(merchant["credentials"]["mercadopago"]["access_token"])
    return sdk


def get_sdk_from_merchant(merchant_id):
    merchant = db.merchants.find_one(bson.ObjectId(merchant_id))
    return mercadopago.SDK(merchant["credentials"]["mercadopago"]["access_token"])


def mercadopago_client(sdk: mercadopago.SDK, cli: dict) -> str:
    """Método que crea un cliente en mercadopago o retorna el id si ya existe

    :param sdk: SDK de mercadopago ya instanciado
    :type sdk: mercadopago.SDK
    :param cli: datos del cliente a crear/buscar
    :type cli: dict
    :return: id del socio en mercadopago
    :rtype: str
    """

    customers_response = sdk.customer().search(filters={'email': cli['email']})
    customers = customers_response['response']

    try:
        street_number = int(cli['domicilio']['altura'])
    except:
        street_number = 1

    mp_customer_data = {
        "email": cli["email"],
        "first_name": cli["nombre"],
        "last_name": cli['apellido'],
        "phone": {"area_code": None, "number": cli['celular']},
        'identification': {'type': 'DNI', 'number': cli['documento']},  # ! Tipo de identificacion hardcodeado
        'address': {
            'street_name': cli['domicilio']['calle'],
            'street_number': street_number,
        }
    }
    if len(customers['results']) > 0:
        print("Cliente ya existente en Mercadopago...")
        print("Email: ", customers['results'][0]['email'])
        print("Mercadopago_id: ", customers['results'][0]['id'])
        # * Cliente ya existente
        customer = customers['results'][0]
    else:
        # * Creación de un cliente
        created_customer = sdk.customer().create(mp_customer_data)
        customer = created_customer['response']
        if 'email' in customer.keys():
            print("Cliente creado en Mercadopago")
            print("Email: ", customer['email'])
            print("Mercadopago_id: ", customer['id'])
        else:
            print('Fallo la creación del cliente en mp')
            print('Cliente con email: ', cli['email'])
            print(customer)
            customer['id'] = 'error'
    return customer['id']


def create_access_token(merchant: dict, marketplace: dict) -> dict:
    """ Método que crea un access token para el marketplace enviado por parámetro

    :param merchant: merchant para el cual se quieren crear credenciales.
    :type merchant: dict
    :param marketplace: marketplace a integrar, con formato de merchant. (Deberia ser el merchant cadena)
    :type marketplace: dict
    :return: diccionario de keys.
    :rtype: dict
    """
    headers = {
        'content-type': 'application/json',
        'authorization': 'Bearer {}'.format(merchant['credentials']['mercadopago']['access_token'])
    }
    
    data = {
        'client_id': marketplace['credentials']['mercadopago']['client_id'],
        'client_secret': marketplace['credentials']['mercadopago']['client_secret'],
        'grant_type': 'authorization_code',
        'code': merchant['credentials']['mp_marketplace']['marketplace_code'],
        'redirect_uri': 'https://www.develop.buen.club/#/merchants' if test else 'https://www.app.buen.club/#/merchants',
    }
    
    response = requests.post('https://api.mercadopago.com/oauth/token', json=data, headers=headers)
    response = response.json()
    pprint(response)
    keys = None
    if 'access_token' in response.keys():
        keys = {
            'access_token': response['access_token'],
            'refresh_token': response['refresh_token'],
            'public_key': response['public_key'],
            'last_refresh_date': datetime.datetime.today()
        }
    return keys


def refresh_token(merchant: dict, marketplace: dict) -> dict:
    """ Método que crea un access token para el marketplace enviado por parámetro

    :param merchant: merchant para el cual se quieren crear credenciales.
    :type merchant: dict
    :param marketplace: marketplace a integrar, con formato de merchant. (Deberia ser el merchant cadena)
    :type marketplace: dict
    :return: diccionario de keys.
    :rtype: dict
    """
    headers = {
        'Content-Type': 'application/json',
        'authorization': 'Bearer {}'.format(merchant['credentials']['mercadopago']['access_token'])
    }
    
    data = {
        "client_id": marketplace['credentials']['mercadopago']['client_id'],
        "client_secret": marketplace['credentials']['mercadopago']['client_secret'],
        "grant_type": "refresh_token",
        "refresh_token": merchant['credentials']['mp_marketplace']['refresh_token'],
    }
    
    response = requests.post('https://api.mercadopago.com/oauth/token', json=data, headers=headers)
    response = response.json()
    pprint(response)
    keys = None
    if 'access_token' in response.keys():
        keys = {
            'access_token': response['access_token'],
            'refresh_token': response['refresh_token'],
            'public_key': response['public_key'],
            'last_refresh_date': datetime.datetime.today()
        }
    return keys


def get_mercadopago_client(sdk, mongo_client):
    customers_response = sdk.customer().search(filters={'id': mongo_client['mercadopago_id']})
    return customers_response['response']



from datetime import datetime, timedelta
from pprint import pprint
from bson import ObjectId
from ..utilities.dates import today_argentina, DIAS_ANTICIPO_DE_PAGO


class Boleta:
    def __init__(self):
        self.member_id = None
        self.date_created = None
        self.original_payment_date = None
        self.source = None
        self.tries = []
        self.status = None
        self.merchant_id = None
        self.charges_detail = None
        self.period = None
        self.plan_id = None

    def create(
        self,
        cliente_id,
        merchant_id,
        plan_id,
        source: str,
        status: str = "error_not_proccesed",
        charges_detail: dict = None,
        opd: datetime = today_argentina(),
        date_created: datetime = today_argentina(),
        **kwargs,
    ):
        for field_id in [cliente_id, merchant_id, plan_id]:
            field_id = ObjectId(field_id) if ObjectId.is_valid(field_id) else None

        self.member_id = cliente_id
        self.date_created = date_created
        self.original_payment_date = opd
        self.source = source
        self.tries = []
        self.status = status
        self.merchant_id = merchant_id
        self.charges_detail = charges_detail if charges_detail else {}
        self.period = None
        self.plan_id = plan_id
        self.__dict__.update(kwargs)

    def push_to_db(self, db):
        if hasattr(self, "_id"):
            result = db.boletas.update_one({"_id": self._id}, {"$set": self.__dict__})
            return self._id

        result = db.boletas.insert_one(self.__dict__)
        self._id = result.inserted_id
        return result.inserted_id

    def import_from_db(self, boleta: dict):
        for key in boleta.keys():
            setattr(self, key, boleta[key])

    def add_try(self, payment_data: dict, card_data: dict = None):
        # Complete with card data
        if card_data:
            for e in ["card_brand", "card_id", "payment_type"]:
                if e in card_data:
                    payment_data[e] = card_data[e]
                    continue
                payment_data[e] = None

        if not payment_data.get("payment_type"):
            payment_data["payment_type"] = payment_data.get("card", {}).get(
                "card_type", None
            )

        # Genera el nuevo try
        new_try = {
            "try_number": len(self.tries) + 1,
            "payment_day": payment_data["payment_day"],
            "payment_type": payment_data["payment_type"],
            "status": payment_data["status"],
            "status_detail": payment_data["status_detail"],
            "card_id": payment_data["card_id"],
            "card_brand": payment_data["card_brand"],
            "payment_id": payment_data["id"],
            "processor": payment_data.get("processor"),
            "date_approved": payment_data.get("date_approved"),
        }
        self.tries.append(new_try)
        self.status = new_try["status"]

    def make_period(self, fecha_de_cobro: datetime, DIAS_DE_ANTICIPO: int = 0):
        if "recurring" in self.source:
            DIAS_DE_ANTICIPO = DIAS_ANTICIPO_DE_PAGO
        fecha_de_cobro += timedelta(days=DIAS_DE_ANTICIPO)
        period = f"{fecha_de_cobro.month}/{fecha_de_cobro.year}"
        self.period = period

    def print(self):
        pprint(self.__dict__)

    def to_dict(self):
        return self.__dict__



from flask import request, jsonify, make_response


def filterQuery(model):
    queryDict = eval(request.args.get('filter'))
    sociosFiltered = model(__raw__=queryDict)
    if len(sociosFiltered) == 0:
        sociosFiltered = 'no result found'
        resp = make_response({'data': {sociosFiltered}})
        resp.headers['Content-Range'] = f"testing"
        return resp